import unittest
from unittest import mock

from ...management.resource_servers import ResourceServers


class TestResourceServers(unittest.TestCase):
    def test_init_with_optionals(self):
        t = ResourceServers(
            domain="domain", token="jwttoken", telemetry=False, timeout=(10, 2)
        )
        self.assertEqual(t.client.options.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get("Auth0-Client", None)
        self.assertEqual(telemetry_header, None)

    @mock.patch("auth0.management.resource_servers.RestClient")
    def test_create(self, mock_rc):
        mock_instance = mock_rc.return_value

        r = ResourceServers(domain="domain", token="jwttoken")

        r.create({"name": "TestApi", "identifier": "https://test.com/api"})

        mock_instance.post.assert_called_with(
            "https://domain/api/v2/resource-servers",
            data={"name": "TestApi", "identifier": "https://test.com/api"},
        )

    @mock.patch("auth0.management.resource_servers.RestClient")
    def test_get_all(self, mock_rc):
        mock_instance = mock_rc.return_value

        r = ResourceServers(domain="domain", token="jwttoken")

        # with default params
        r.get_all()

        mock_instance.get.assert_called_with(
            "https://domain/api/v2/resource-servers",
            params={"page": None, "per_page": None, "include_totals": "false"},
        )

        # with pagination params
        r.get_all(page=3, per_page=27, include_totals=True)

        mock_instance.get.assert_called_with(
            "https://domain/api/v2/resource-servers",
            params={"page": 3, "per_page": 27, "include_totals": "true"},
        )

    @mock.patch("auth0.management.resource_servers.RestClient")
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value

        r = ResourceServers(domain="domain", token="jwttoken")

        r.get("some_id")

        mock_instance.get.assert_called_with(
            "https://domain/api/v2/resource-servers/some_id"
        )

    @mock.patch("auth0.management.resource_servers.RestClient")
    def test_delete(self, mock_rc):
        mock_instance = mock_rc.return_value

        r = ResourceServers(domain="domain", token="jwttoken")

        r.delete("some_id")

        mock_instance.delete.assert_called_with(
            "https://domain/api/v2/resource-servers/some_id"
        )

    @mock.patch("auth0.management.resource_servers.RestClient")
    def test_update(self, mock_rc):
        mock_instance = mock_rc.return_value

        r = ResourceServers(domain="domain", token="jwttoken")

        r.update("some_id", {"name": "TestApi2", "identifier": "https://test.com/api2"})

        mock_instance.patch.assert_called_with(
            "https://domain/api/v2/resource-servers/some_id",
            data={"name": "TestApi2", "identifier": "https://test.com/api2"},
        )



import unittest
from unittest import mock

from ...management.attack_protection import AttackProtection


class TestAttackProtection(unittest.TestCase):
    def test_init_with_optionals(self):
        t = AttackProtection(
            domain="domain", token="jwttoken", telemetry=False, timeout=(10, 2)
        )
        self.assertEqual(t.client.options.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get("Auth0-Client", None)
        self.assertEqual(telemetry_header, None)

    @mock.patch("auth0.management.attack_protection.RestClient")
    def test_get_breached_password_detection(self, mock_rc):
        mock_instance = mock_rc.return_value
        mock_instance.get.return_value = {}

        ap = AttackProtection(domain="domain", token="jwttoken")
        ap.get_breached_password_detection()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/attack-protection/breached-password-detection",
            args[0],
        )

    @mock.patch("auth0.management.attack_protection.RestClient")
    def test_update_breached_password_detection(self, mock_rc):
        mock_instance = mock_rc.return_value
        mock_instance.patch.return_value = {}

        c = AttackProtection(domain="domain", token="jwttoken")
        c.update_breached_password_detection({"a": "b", "c": "d"})

        mock_instance.patch.assert_called_with(
            "https://domain/api/v2/attack-protection/breached-password-detection",
            data={"a": "b", "c": "d"},
        )

    @mock.patch("auth0.management.attack_protection.RestClient")
    def test_get_brute_force_protection(self, mock_rc):
        mock_instance = mock_rc.return_value
        mock_instance.get.return_value = {}

        ap = AttackProtection(domain="domain", token="jwttoken")
        ap.get_brute_force_protection()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/attack-protection/brute-force-protection", args[0]
        )

    @mock.patch("auth0.management.attack_protection.RestClient")
    def test_update_brute_force_protection(self, mock_rc):
        mock_instance = mock_rc.return_value
        mock_instance.patch.return_value = {}

        c = AttackProtection(domain="domain", token="jwttoken")
        c.update_brute_force_protection({"a": "b", "c": "d"})

        mock_instance.patch.assert_called_with(
            "https://domain/api/v2/attack-protection/brute-force-protection",
            data={"a": "b", "c": "d"},
        )

    @mock.patch("auth0.management.attack_protection.RestClient")
    def test_get_suspicious_ip_throttling(self, mock_rc):
        mock_instance = mock_rc.return_value
        mock_instance.get.return_value = {}

        ap = AttackProtection(domain="domain", token="jwttoken")
        ap.get_suspicious_ip_throttling()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/attack-protection/suspicious-ip-throttling", args[0]
        )

    @mock.patch("auth0.management.attack_protection.RestClient")
    def test_update_suspicious_ip_throttling(self, mock_rc):
        mock_instance = mock_rc.return_value
        mock_instance.patch.return_value = {}

        c = AttackProtection(domain="domain", token="jwttoken")
        c.update_suspicious_ip_throttling({"a": "b", "c": "d"})

        mock_instance.patch.assert_called_with(
            "https://domain/api/v2/attack-protection/suspicious-ip-throttling",
            data={"a": "b", "c": "d"},
        )



import time
import unittest
from unittest.mock import ANY

import jwt
from aioresponses import aioresponses
from callee import Attrs
from cryptography.hazmat.primitives import serialization

from .. import TokenValidationError
from ..authentication.async_token_verifier import (
    AsyncAsymmetricSignatureVerifier,
    AsyncJwksFetcher,
    AsyncTokenVerifier,
)
from ..test.authentication.test_token_verifier import (
    JWKS_RESPONSE_MULTIPLE_KEYS,
    JWKS_RESPONSE_SINGLE_KEY,
    RSA_PUB_KEY_1_JWK,
    RSA_PUB_KEY_1_PEM,
    RSA_PUB_KEY_2_PEM,
)
from .test_asyncify import get_callback

JWKS_URI = "https://example.auth0.com/.well-known/jwks.json"

PRIVATE_KEY = """-----BEGIN RSA PRIVATE KEY-----
MIICXAIBAAKBgQDfytWVSk/4Z6rNu8UZ7C4tnU9x0vj5FCaj4awKZlxVgOR1Kcen
QqDOxJdrXXanTBJbZwh8pk+HpWvqDVgVmKhnt+OkgF//hIXZoJMhDOFVzX504kiZ
cu3bu7kFs+PUfKw5s59tmETFPseA/fIrad9YXHisMkNmPWhuKYJ3WfZAaQIDAQAB
AoGADPSfHL9qlcTanIJsTK3hln5u5PYDt9e0zPP5k7iNS93kW+wJROOUj6PN6EdG
4TSEM4ppcV3naMDo2GnhWY624P6LUB+CbDFzjQKq805vrxJuFnq50blscwVK/ffP
kODBm/gwk+FaliRpQTDAAPWkKbkRfkmPx4JMEmTDBQ45diECQQDxw3qp2+wa5WP5
9w7AYrDPq4Fd6gIFcmxracROUcdhhMmVHKA9DzTWY46cSoWZoChYhQhhyj8dlP8q
El8aevN9AkEA7PhxcNyff8aehqEQ/Z38bm3P+GgB9EkRinjesba2CqhEI5okzvb7
OIYdszgQUBqGKlST0a7s9KuTpd7moyy8XQJAY8hjk0HCxCMTTXMLspnJEh1eKo3P
wcHFP9wKeqzEFtrAfHuxIyJok2fJz3XuiEaTAF3/5KSdwi7h1dJ5UCuY3QJAM9rF
0CGnEWngJKu4MRdSNsP232+7Bb67hOagLJlDyp85keTYKyXmoV7PvvkEsNKtCzRI
yHiTx5KIE6LsK0bNzQJBAMV+1KyI8ua1XmqLDaOexvBPM86HnuP+8u5CthgrXyGm
nh9gurwbs/lBRYV/d4XBLj+dzHb2zC0Jo7u96wrOObw=
-----END RSA PRIVATE KEY-----"""

PUBLIC_KEY = {
    "kty": "RSA",
    "e": "AQAB",
    "kid": "kid-1",
    "n": "38rVlUpP-GeqzbvFGewuLZ1PcdL4-RQmo-GsCmZcVYDkdSnHp0KgzsSXa112p0wSW2cIfKZPh6Vr6g1YFZioZ7fjpIBf_4SF2aCTIQzhVc1-dOJImXLt27u5BbPj1HysObOfbZhExT7HgP3yK2nfWFx4rDJDZj1obimCd1n2QGk",
}


def get_pem_bytes(rsa_public_key):
    return rsa_public_key.public_bytes(
        serialization.Encoding.PEM, serialization.PublicFormat.SubjectPublicKeyInfo
    )


@unittest.skipIf(
    not hasattr(unittest, "IsolatedAsyncioTestCase"),
    "python 3.7 doesn't have IsolatedAsyncioTestCase",
)
class TestAsyncAsymmetricSignatureVerifier(
    getattr(unittest, "IsolatedAsyncioTestCase", object)
):
    @aioresponses()
    async def test_async_asymmetric_verifier_fetches_key(self, mocked):
        callback, mock = get_callback(200, JWKS_RESPONSE_SINGLE_KEY)
        mocked.get(JWKS_URI, callback=callback)

        verifier = AsyncAsymmetricSignatureVerifier(JWKS_URI)

        key = await verifier._fetch_key("test-key-1")

        self.assertEqual(get_pem_bytes(key), RSA_PUB_KEY_1_PEM)


@unittest.skipIf(
    not hasattr(unittest, "IsolatedAsyncioTestCase"),
    "python 3.7 doesn't have IsolatedAsyncioTestCase",
)
class TestAsyncJwksFetcher(getattr(unittest, "IsolatedAsyncioTestCase", object)):
    @aioresponses()
    @unittest.mock.patch(
        "auth0.authentication.token_verifier.time.time", return_value=0
    )
    async def test_async_get_jwks_json_twice_on_cache_expired(
        self, mocked, mocked_time
    ):
        fetcher = AsyncJwksFetcher(JWKS_URI, cache_ttl=100)

        callback, mock = get_callback(200, JWKS_RESPONSE_SINGLE_KEY)
        mocked.get(JWKS_URI, callback=callback)
        mocked.get(JWKS_URI, callback=callback)

        key_1 = await fetcher.get_key("test-key-1")
        expected_key_1_pem = get_pem_bytes(key_1)
        self.assertEqual(expected_key_1_pem, RSA_PUB_KEY_1_PEM)

        mock.assert_called_with(
            Attrs(path="/.well-known/jwks.json"),
            allow_redirects=True,
            params=None,
            headers=ANY,
            timeout=ANY,
        )
        self.assertEqual(mock.call_count, 1)

        mocked_time.return_value = 200

        # 2 seconds has passed, cache should be expired
        key_1 = await fetcher.get_key("test-key-1")
        expected_key_1_pem = get_pem_bytes(key_1)
        self.assertEqual(expected_key_1_pem, RSA_PUB_KEY_1_PEM)

        mock.assert_called_with(
            Attrs(path="/.well-known/jwks.json"),
            allow_redirects=True,
            params=None,
            headers=ANY,
            timeout=ANY,
        )
        self.assertEqual(mock.call_count, 2)

    @aioresponses()
    async def test_async_get_jwks_json_once_on_cache_hit(self, mocked):
        fetcher = AsyncJwksFetcher(JWKS_URI, cache_ttl=1)

        callback, mock = get_callback(200, JWKS_RESPONSE_MULTIPLE_KEYS)
        mocked.get(JWKS_URI, callback=callback)
        mocked.get(JWKS_URI, callback=callback)

        key_1 = await fetcher.get_key("test-key-1")
        key_2 = await fetcher.get_key("test-key-2")
        expected_key_1_pem = get_pem_bytes(key_1)
        expected_key_2_pem = get_pem_bytes(key_2)
        self.assertEqual(expected_key_1_pem, RSA_PUB_KEY_1_PEM)
        self.assertEqual(expected_key_2_pem, RSA_PUB_KEY_2_PEM)

        mock.assert_called_with(
            Attrs(path="/.well-known/jwks.json"),
            allow_redirects=True,
            params=None,
            headers=ANY,
            timeout=ANY,
        )
        self.assertEqual(mock.call_count, 1)

    @aioresponses()
    async def test_async_fetches_jwks_json_forced_on_cache_miss(self, mocked):
        fetcher = AsyncJwksFetcher(JWKS_URI, cache_ttl=1)

        callback, mock = get_callback(200, {"keys": [RSA_PUB_KEY_1_JWK]})
        mocked.get(JWKS_URI, callback=callback)

        # Triggers the first call
        key_1 = await fetcher.get_key("test-key-1")
        expected_key_1_pem = get_pem_bytes(key_1)
        self.assertEqual(expected_key_1_pem, RSA_PUB_KEY_1_PEM)

        mock.assert_called_with(
            Attrs(path="/.well-known/jwks.json"),
            allow_redirects=True,
            params=None,
            headers=ANY,
            timeout=ANY,
        )
        self.assertEqual(mock.call_count, 1)

        callback, mock = get_callback(200, JWKS_RESPONSE_MULTIPLE_KEYS)
        mocked.get(JWKS_URI, callback=callback)

        # Triggers the second call
        key_2 = await fetcher.get_key("test-key-2")
        expected_key_2_pem = get_pem_bytes(key_2)
        self.assertEqual(expected_key_2_pem, RSA_PUB_KEY_2_PEM)

        mock.assert_called_with(
            Attrs(path="/.well-known/jwks.json"),
            allow_redirects=True,
            params=None,
            headers=ANY,
            timeout=ANY,
        )
        self.assertEqual(mock.call_count, 1)

    @aioresponses()
    async def test_async_fetches_jwks_json_once_on_cache_miss(self, mocked):
        fetcher = AsyncJwksFetcher(JWKS_URI, cache_ttl=1)

        callback, mock = get_callback(200, JWKS_RESPONSE_SINGLE_KEY)
        mocked.get(JWKS_URI, callback=callback)

        with self.assertRaises(Exception) as err:
            await fetcher.get_key("missing-key")

        mock.assert_called_with(
            Attrs(path="/.well-known/jwks.json"),
            allow_redirects=True,
            params=None,
            headers=ANY,
            timeout=ANY,
        )
        self.assertEqual(
            str(err.exception), 'RSA Public Key with ID "missing-key" was not found.'
        )
        self.assertEqual(mock.call_count, 1)

    @aioresponses()
    async def test_async_fails_to_fetch_jwks_json_after_retrying_twice(self, mocked):
        fetcher = AsyncJwksFetcher(JWKS_URI, cache_ttl=1)

        callback, mock = get_callback(500, {})
        mocked.get(JWKS_URI, callback=callback)
        mocked.get(JWKS_URI, callback=callback)

        with self.assertRaises(Exception) as err:
            await fetcher.get_key("id1")

        mock.assert_called_with(
            Attrs(path="/.well-known/jwks.json"),
            allow_redirects=True,
            params=None,
            headers=ANY,
            timeout=ANY,
        )
        self.assertEqual(
            str(err.exception), 'RSA Public Key with ID "id1" was not found.'
        )
        self.assertEqual(mock.call_count, 2)


@unittest.skipIf(
    not hasattr(unittest, "IsolatedAsyncioTestCase"),
    "python 3.7 doesn't have IsolatedAsyncioTestCase",
)
class TestAsyncTokenVerifier(getattr(unittest, "IsolatedAsyncioTestCase", object)):
    @aioresponses()
    async def test_RS256_token_signature_passes(self, mocked):
        callback, mock = get_callback(200, {"keys": [PUBLIC_KEY]})
        mocked.get(JWKS_URI, callback=callback)

        issuer = "https://tokens-test.auth0.com/"
        audience = "tokens-test-123"
        token = jwt.encode(
            {
                "iss": issuer,
                "sub": "auth0|123456789",
                "aud": audience,
                "exp": int(time.time()) + 86400,
                "iat": int(time.time()),
            },
            PRIVATE_KEY,
            algorithm="RS256",
            headers={"kid": "kid-1"},
        )

        tv = AsyncTokenVerifier(
            signature_verifier=AsyncAsymmetricSignatureVerifier(JWKS_URI),
            issuer=issuer,
            audience=audience,
        )
        payload = await tv.verify(token)
        self.assertEqual(payload["sub"], "auth0|123456789")

    @aioresponses()
    async def test_RS256_token_signature_fails(self, mocked):
        callback, mock = get_callback(
            200, {"keys": [RSA_PUB_KEY_1_JWK]}
        )  # different pub key
        mocked.get(JWKS_URI, callback=callback)

        issuer = "https://tokens-test.auth0.com/"
        audience = "tokens-test-123"
        token = jwt.encode(
            {
                "iss": issuer,
                "sub": "auth0|123456789",
                "aud": audience,
                "exp": int(time.time()) + 86400,
                "iat": int(time.time()),
            },
            PRIVATE_KEY,
            algorithm="RS256",
            headers={"kid": "test-key-1"},
        )

        tv = AsyncTokenVerifier(
            signature_verifier=AsyncAsymmetricSignatureVerifier(JWKS_URI),
            issuer=issuer,
            audience=audience,
        )

        with self.assertRaises(TokenValidationError) as err:
            await tv.verify(token)
        self.assertEqual(str(err.exception), "Invalid token signature.")



from auth0.rest import RestClient, RestClientOptions

from .client_authentication import add_client_authentication

UNKNOWN_ERROR = "a0.sdk.internal.unknown"


class AuthenticationBase:
    """Base authentication object providing simple REST methods.

    Args:
        domain (str): The domain of your Auth0 tenant
        client_id (str): Your application's client ID
        client_secret (str, optional): Your application's client secret
        client_assertion_signing_key (str, optional): Private key used to sign the client assertion JWT.
        client_assertion_signing_alg (str, optional): Algorithm used to sign the client assertion JWT (defaults to 'RS256').
        telemetry (bool, optional): Enable or disable telemetry (defaults to True)
        timeout (float or tuple, optional): Change the requests connect and read timeout. Pass a tuple to specify both values separately or a float to set both to it. (defaults to 5.0 for both)
        protocol (str, optional): Useful for testing. (defaults to 'https')
    """

    def __init__(
        self,
        domain,
        client_id,
        client_secret=None,
        client_assertion_signing_key=None,
        client_assertion_signing_alg=None,
        telemetry=True,
        timeout=5.0,
        protocol="https",
    ):
        self.domain = domain
        self.client_id = client_id
        self.client_secret = client_secret
        self.client_assertion_signing_key = client_assertion_signing_key
        self.client_assertion_signing_alg = client_assertion_signing_alg
        self.protocol = protocol
        self.client = RestClient(
            None,
            options=RestClientOptions(telemetry=telemetry, timeout=timeout, retries=0),
        )

    def _add_client_authentication(self, payload):
        return add_client_authentication(
            payload,
            self.domain,
            self.client_id,
            self.client_secret,
            self.client_assertion_signing_key,
            self.client_assertion_signing_alg,
        )

    def post(self, url, data=None, headers=None):
        return self.client.post(url, data=data, headers=headers)

    def authenticated_post(self, url, data=None, headers=None):
        return self.client.post(
            url, data=self._add_client_authentication(data), headers=headers
        )

    def get(self, url, params=None, headers=None):
        return self.client.get(url, params, headers)



from ..rest import RestClient


class Organizations:
    """Auth0 organizations endpoints

    Args:
        domain (str): Your Auth0 domain, e.g: 'username.auth0.com'

        token (str): Management API v2 Token

        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)

        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)

        rest_options (RestClientOptions): Pass an instance of
            RestClientOptions to configure additional RestClient
            options, such as rate-limit retries.
            (defaults to None)
    """

    def __init__(
        self,
        domain,
        token,
        telemetry=True,
        timeout=5.0,
        protocol="https",
        rest_options=None,
    ):
        self.domain = domain
        self.protocol = protocol
        self.client = RestClient(
            jwt=token, telemetry=telemetry, timeout=timeout, options=rest_options
        )

    def _url(self, *args):
        url = f"{self.protocol}://{self.domain}/api/v2/organizations"
        for p in args:
            if p is not None:
                url = f"{url}/{p}"
        return url

    # Organizations
    def all_organizations(
        self, page=None, per_page=None, include_totals=True, from_param=None, take=None
    ):
        """Retrieves a list of all the organizations.

        Args:
            page (int): The result's page number (zero based). When not set,
                the default value is up to the server.

            per_page (int, optional): The amount of entries per page. When not set,
                the default value is up to the server.

            include_totals (bool, optional): True if the query summary is
                to be included in the result, False otherwise. Defaults to True.

            from_param (str, optional): Checkpoint Id from which to begin retrieving results.
                You can limit the number of entries using the take parameter.

            take (int, optional): The total amount of entries to retrieve when
                using the from parameter. When not set, the default value is up to the server.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_organizations
        """

        params = {
            "page": page,
            "per_page": per_page,
            "include_totals": str(include_totals).lower(),
            "from": from_param,
            "take": take,
        }

        return self.client.get(self._url(), params=params)

    def get_organization_by_name(self, name=None):
        """Retrieves an organization given its name.

        Args:
           name (str): The name of the organization to retrieve.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_name_by_name
        """
        params = {}

        return self.client.get(self._url("name", name), params=params)

    def get_organization(self, id):
        """Retrieves an organization by its ID.

        Args:
           id (str): Id of organization to retrieve.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_organizations_by_id
        """
        params = {}

        return self.client.get(self._url(id), params=params)

    def create_organization(self, body):
        """Create a new organization.

        Args:
           body (dict): Attributes for the new organization.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/post_organizations
        """

        return self.client.post(self._url(), data=body)

    def update_organization(self, id, body):
        """Modifies an organization.

        Args:
           id (str): the ID of the organization.

           body (dict): Attributes to modify.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/patch_organizations_by_id
        """

        return self.client.patch(self._url(id), data=body)

    def delete_organization(self, id):
        """Deletes an organization and all its related assets.

        Args:
           id (str): Id of organization to delete.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/delete_organizations_by_id
        """

        return self.client.delete(self._url(id))

    # Organization Connections
    def all_organization_connections(self, id, page=None, per_page=None):
        """Retrieves a list of all the organization connections.

        Args:
           id (str): the ID of the organization.

           page (int): The result's page number (zero based). When not set,
              the default value is up to the server.

           per_page (int, optional): The amount of entries per page. When not set,
              the default value is up to the server.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_enabled_connections
        """
        params = {"page": page, "per_page": per_page}
        return self.client.get(self._url(id, "enabled_connections"), params=params)

    def get_organization_connection(self, id, connection_id):
        """Retrieves an organization connection by its ID.

        Args:
           id (str): the ID of the organization.

           connection_id (str): the ID of the connection.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_enabled_connections_by_connectionId
        """
        params = {}

        return self.client.get(
            self._url(id, "enabled_connections", connection_id), params=params
        )

    def create_organization_connection(self, id, body):
        """Adds a connection to an organization.

        Args:
           id (str): the ID of the organization.

           body (dict): Attributes for the connection to add.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/post_enabled_connections
        """

        return self.client.post(self._url(id, "enabled_connections"), data=body)

    def update_organization_connection(self, id, connection_id, body):
        """Modifies an organization.

        Args:
           id (str): the ID of the organization.

           connection_id (str): the ID of the connection to update.

           body (dict): Attributes to modify.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/patch_enabled_connections_by_connectionId
        """

        return self.client.patch(
            self._url(id, "enabled_connections", connection_id), data=body
        )

    def delete_organization_connection(self, id, connection_id):
        """Deletes a connection from the given organization.

        Args:
           id (str): Id of organization.

           connection_id (str): the ID of the connection to delete.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/delete_enabled_connections_by_connectionId
        """

        return self.client.delete(self._url(id, "enabled_connections", connection_id))

    # Organization Members
    def all_organization_members(
        self,
        id,
        page=None,
        per_page=None,
        include_totals=True,
        from_param=None,
        take=None,
    ):
        """Retrieves a list of all the organization members.

        Args:
            id (str): the ID of the organization.

            page (int): The result's page number (zero based). When not set,
                the default value is up to the server.

            per_page (int, optional): The amount of entries per page. When not set,
                the default value is up to the server.

            include_totals (bool, optional): True if the query summary is
                to be included in the result, False otherwise. Defaults to True.

            from_param (str, optional): Checkpoint Id from which to begin retrieving results.
                You can limit the number of entries using the take parameter.

            take (int, optional): The total amount of entries to retrieve when
                using the from parameter. When not set, the default value is up to the server.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_members
        """

        params = {
            "page": page,
            "per_page": per_page,
            "include_totals": str(include_totals).lower(),
            "from": from_param,
            "take": take,
        }

        return self.client.get(self._url(id, "members"), params=params)

    def create_organization_members(self, id, body):
        """Adds members to an organization.

        Args:
           id (str): the ID of the organization.

           body (dict): Attributes from the members to add.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/post_members
        """

        return self.client.post(self._url(id, "members"), data=body)

    def delete_organization_members(self, id, body):
        """Deletes members from the given organization.

        Args:
           id (str): Id of organization.

           body (dict): Attributes from the members to delete

        See: https://auth0.com/docs/api/management/v2#!/Organizations/delete_members
        """

        return self.client.delete(self._url(id, "members"), data=body)

    # Organization Member Roles
    def all_organization_member_roles(self, id, user_id, page=None, per_page=None):
        """Retrieves a list of all the roles from the given organization member.

        Args:
           id (str): the ID of the organization.

           user_id (str): the ID of the user member of the organization.

           page (int): The result's page number (zero based). When not set,
              the default value is up to the server.

           per_page (int, optional): The amount of entries per page. When not set,
              the default value is up to the server.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_organization_member_roles
        """
        params = {"page": page, "per_page": per_page}
        return self.client.get(
            self._url(id, "members", user_id, "roles"), params=params
        )

    def create_organization_member_roles(self, id, user_id, body):
        """Adds roles to a member of an organization.

        Args:
           id (str): the ID of the organization.

           user_id (str): the ID of the user member of the organization.

           body (dict): Attributes from the members to add.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/post_organization_member_roles
        """

        return self.client.post(self._url(id, "members", user_id, "roles"), data=body)

    def delete_organization_member_roles(self, id, user_id, body):
        """Deletes roles from a member of an organization.

        Args:
           id (str): Id of organization.

           user_id (str): the ID of the user member of the organization.

           body (dict): Attributes from the members to delete

        See: https://auth0.com/docs/api/management/v2#!/Organizations/delete_organization_member_roles
        """

        return self.client.delete(self._url(id, "members", user_id, "roles"), data=body)

    # Organization Invitations
    def all_organization_invitations(
        self,
        id,
        page=None,
        per_page=None,
        include_totals=False,
    ):
        """Retrieves a list of all the organization invitations.

        Args:
           id (str): the ID of the organization.

           page (int): The result's page number (zero based). When not set,
              the default value is up to the server.

           per_page (int, optional): The amount of entries per page. When not set,
              the default value is up to the server.

           include_totals (bool, optional): True if the query summary is
              to be included in the result, False otherwise. Defaults to False.
              NOTE: returns start and limit, total count is not yet supported

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_invitations
        """
        params = {
            "page": page,
            "per_page": per_page,
            "include_totals": str(include_totals).lower(),
        }

        return self.client.get(self._url(id, "invitations"), params=params)

    def get_organization_invitation(self, id, invitaton_id):
        """Retrieves an organization invitation by its ID.

        Args:
           id (str): the ID of the organization.

           invitaton_id (str): the ID of the invitation.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/get_invitations_by_invitation_id
        """
        params = {}

        return self.client.get(
            self._url(id, "invitations", invitaton_id), params=params
        )

    def create_organization_invitation(self, id, body):
        """Create an invitation to an organization.

        Args:
           id (str): the ID of the organization.

           body (dict): Attributes for the invitation to create.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/post_invitations
        """

        return self.client.post(self._url(id, "invitations"), data=body)

    def delete_organization_invitation(self, id, invitation_id):
        """Deletes an invitation from the given organization.

        Args:
           id (str): Id of organization.

           invitation_id (str): the ID of the invitation to delete.

        See: https://auth0.com/docs/api/management/v2#!/Organizations/delete_invitations_by_invitation_id
        """

        return self.client.delete(self._url(id, "invitations", invitation_id))



import unittest
from unittest import mock

from ...management.organizations import Organizations


class TestOrganizations(unittest.TestCase):
    def test_init_with_optionals(self):
        t = Organizations(
            domain="domain", token="jwttoken", telemetry=False, timeout=(10, 2)
        )
        self.assertEqual(t.client.options.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get("Auth0-Client", None)
        self.assertEqual(telemetry_header, None)

    # Organizations
    @mock.patch("auth0.management.organizations.RestClient")
    def test_all_organizations(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")

        # Default parameters are requested
        c.all_organizations()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/organizations", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "page": None,
                "per_page": None,
                "include_totals": "true",
                "from": None,
                "take": None,
            },
        )

        # Basic pagination
        c.all_organizations(page=7, per_page=25, include_totals=False)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/organizations", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "page": 7,
                "per_page": 25,
                "include_totals": "false",
                "from": None,
                "take": None,
            },
        )

        # Checkpoint pagination
        c.all_organizations(from_param=8675309, take=75)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/organizations", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "from": 8675309,
                "take": 75,
                "page": None,
                "per_page": None,
                "include_totals": "true",
            },
        )

    @mock.patch("auth0.management.organizations.RestClient")
    def test_get_organization_by_name(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")
        c.get_organization_by_name("test-org")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/organizations/name/test-org", args[0])
        self.assertEqual(kwargs["params"], {})

    @mock.patch("auth0.management.organizations.RestClient")
    def test_get_organization(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")
        c.get_organization("org123")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/organizations/org123", args[0])
        self.assertEqual(kwargs["params"], {})

    @mock.patch("auth0.management.organizations.RestClient")
    def test_create_organization(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")
        c.create_organization({"a": "b", "c": "d"})

        mock_instance.post.assert_called_with(
            "https://domain/api/v2/organizations", data={"a": "b", "c": "d"}
        )

    @mock.patch("auth0.management.organizations.RestClient")
    def test_update_organization(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")
        c.update_organization("this-id", {"a": "b", "c": "d"})

        args, kwargs = mock_instance.patch.call_args

        self.assertEqual("https://domain/api/v2/organizations/this-id", args[0])
        self.assertEqual(kwargs["data"], {"a": "b", "c": "d"})

    @mock.patch("auth0.management.organizations.RestClient")
    def test_delete_organization(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")
        c.delete_organization("this-id")

        mock_instance.delete.assert_called_with(
            "https://domain/api/v2/organizations/this-id"
        )

    # Organization Connections
    @mock.patch("auth0.management.organizations.RestClient")
    def test_all_organization_connections(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")

        # Default parameters are requested
        c.all_organization_connections("test-org")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/organizations/test-org/enabled_connections", args[0]
        )
        self.assertEqual(kwargs["params"], {"page": None, "per_page": None})

        # Specific pagination
        c.all_organization_connections("test-org", page=7, per_page=25)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/organizations/test-org/enabled_connections", args[0]
        )
        self.assertEqual(kwargs["params"], {"page": 7, "per_page": 25})

    @mock.patch("auth0.management.organizations.RestClient")
    def test_get_organization_connection(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")
        c.get_organization_connection("test-org", "test-con")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/organizations/test-org/enabled_connections/test-con",
            args[0],
        )
        self.assertEqual(kwargs["params"], {})

    @mock.patch("auth0.management.organizations.RestClient")
    def test_create_organization_connection(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")
        c.create_organization_connection("test-org", {"a": "b", "c": "d"})

        mock_instance.post.assert_called_with(
            "https://domain/api/v2/organizations/test-org/enabled_connections",
            data={"a": "b", "c": "d"},
        )

    @mock.patch("auth0.management.organizations.RestClient")
    def test_update_organization_connection(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")
        c.update_organization_connection("test-org", "test-con", {"a": "b", "c": "d"})

        args, kwargs = mock_instance.patch.call_args

        self.assertEqual(
            "https://domain/api/v2/organizations/test-org/enabled_connections/test-con",
            args[0],
        )
        self.assertEqual(kwargs["data"], {"a": "b", "c": "d"})

    @mock.patch("auth0.management.organizations.RestClient")
    def test_delete_organization_connection(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")
        c.delete_organization_connection("test-org", "test-con")

        mock_instance.delete.assert_called_with(
            "https://domain/api/v2/organizations/test-org/enabled_connections/test-con"
        )

    # Organization Members
    @mock.patch("auth0.management.organizations.RestClient")
    def test_all_organization_members(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")

        # Default parameters are requested
        c.all_organization_members("test-org")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/organizations/test-org/members", args[0]
        )
        self.assertEqual(
            kwargs["params"],
            {
                "page": None,
                "per_page": None,
                "include_totals": "true",
                "from": None,
                "take": None,
            },
        )

        # Specific pagination
        c.all_organization_members(
            "test-org", page=7, per_page=25, include_totals=False
        )

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/organizations/test-org/members", args[0]
        )
        self.assertEqual(
            kwargs["params"],
            {
                "page": 7,
                "per_page": 25,
                "include_totals": "false",
                "from": None,
                "take": None,
            },
        )

        # Checkpoint pagination
        c.all_organization_members("test-org", from_param=8675309, take=75)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/organizations/test-org/members", args[0]
        )
        self.assertEqual(
            kwargs["params"],
            {
                "from": 8675309,
                "take": 75,
                "page": None,
                "per_page": None,
                "include_totals": "true",
            },
        )

    @mock.patch("auth0.management.organizations.RestClient")
    def test_create_organization_members(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")
        c.create_organization_members("test-org", {"a": "b", "c": "d"})

        mock_instance.post.assert_called_with(
            "https://domain/api/v2/organizations/test-org/members",
            data={"a": "b", "c": "d"},
        )

    @mock.patch("auth0.management.organizations.RestClient")
    def test_delete_organization_members(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")
        c.delete_organization_members("test-org", {"a": "b", "c": "d"})

        mock_instance.delete.assert_called_with(
            "https://domain/api/v2/organizations/test-org/members",
            data={"a": "b", "c": "d"},
        )

    # Organization Member Roles
    @mock.patch("auth0.management.organizations.RestClient")
    def test_all_organization_member_roles(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")

        # Default parameters are requested
        c.all_organization_member_roles("test-org", "test-user")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/organizations/test-org/members/test-user/roles",
            args[0],
        )
        self.assertEqual(kwargs["params"], {"page": None, "per_page": None})

        # Specific pagination
        c.all_organization_member_roles("test-org", "test-user", page=7, per_page=25)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/organizations/test-org/members/test-user/roles",
            args[0],
        )
        self.assertEqual(kwargs["params"], {"page": 7, "per_page": 25})

    @mock.patch("auth0.management.organizations.RestClient")
    def test_create_organization_member_roles(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")
        c.create_organization_member_roles(
            "test-org", "test-user", {"a": "b", "c": "d"}
        )

        mock_instance.post.assert_called_with(
            "https://domain/api/v2/organizations/test-org/members/test-user/roles",
            data={"a": "b", "c": "d"},
        )

    @mock.patch("auth0.management.organizations.RestClient")
    def test_delete_organization_member_roles(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")
        c.delete_organization_member_roles(
            "test-org", "test-user", {"a": "b", "c": "d"}
        )

        mock_instance.delete.assert_called_with(
            "https://domain/api/v2/organizations/test-org/members/test-user/roles",
            data={"a": "b", "c": "d"},
        )

    # Organization Invitations
    @mock.patch("auth0.management.organizations.RestClient")
    def test_all_organization_invitations(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")

        # Default parameters are requested
        c.all_organization_invitations("test-org")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/organizations/test-org/invitations", args[0]
        )
        self.assertEqual(
            kwargs["params"],
            {
                "page": None,
                "per_page": None,
                "include_totals": "false",
            },
        )

        # Specific pagination
        c.all_organization_invitations("test-org", page=7, per_page=25)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/organizations/test-org/invitations", args[0]
        )
        self.assertEqual(
            kwargs["params"],
            {
                "page": 7,
                "per_page": 25,
                "include_totals": "false",
            },
        )

        # Return paged collection with paging properties
        c.all_organization_invitations(
            "test-org", page=7, per_page=25, include_totals=True
        )

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/organizations/test-org/invitations", args[0]
        )
        self.assertEqual(
            kwargs["params"],
            {
                "page": 7,
                "per_page": 25,
                "include_totals": "true",
            },
        )

    @mock.patch("auth0.management.organizations.RestClient")
    def test_get_organization_invitation(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")
        c.get_organization_invitation("test-org", "test-inv")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/organizations/test-org/invitations/test-inv", args[0]
        )
        self.assertEqual(kwargs["params"], {})

    @mock.patch("auth0.management.organizations.RestClient")
    def test_create_organization_invitation(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")
        c.create_organization_invitation("test-org", {"a": "b", "c": "d"})

        mock_instance.post.assert_called_with(
            "https://domain/api/v2/organizations/test-org/invitations",
            data={"a": "b", "c": "d"},
        )

    @mock.patch("auth0.management.organizations.RestClient")
    def test_delete_organization_invitation(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Organizations(domain="domain", token="jwttoken")
        c.delete_organization_invitation("test-org", "test-inv")

        mock_instance.delete.assert_called_with(
            "https://domain/api/v2/organizations/test-org/invitations/test-inv"
        )



import unittest
from unittest import mock

from ...management.user_blocks import UserBlocks


class TestUserBlocks(unittest.TestCase):
    def test_init_with_optionals(self):
        t = UserBlocks(
            domain="domain", token="jwttoken", telemetry=False, timeout=(10, 2)
        )
        self.assertEqual(t.client.options.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get("Auth0-Client", None)
        self.assertEqual(telemetry_header, None)

    @mock.patch("auth0.management.user_blocks.RestClient")
    def test_get_by_identifier(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = UserBlocks(domain="domain", token="jwttoken")

        u.get_by_identifier("some_identifier")

        mock_instance.get.assert_called_with(
            "https://domain/api/v2/user-blocks",
            params={"identifier": "some_identifier"},
        )

    @mock.patch("auth0.management.user_blocks.RestClient")
    def test_unblock_by_identifier(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = UserBlocks(domain="domain", token="jwttoken")

        u.unblock_by_identifier("test@test.com")

        mock_instance.delete.assert_called_with(
            "https://domain/api/v2/user-blocks", params={"identifier": "test@test.com"}
        )

    @mock.patch("auth0.management.user_blocks.RestClient")
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = UserBlocks(domain="domain", token="jwttoken")

        u.get("auth0|584ad3c228be27504a2c80d5")

        mock_instance.get.assert_called_with(
            "https://domain/api/v2/user-blocks/auth0|584ad3c228be27504a2c80d5"
        )

    @mock.patch("auth0.management.user_blocks.RestClient")
    def test_unblock(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = UserBlocks(domain="domain", token="jwttoken")

        u.unblock("auth0|584ad3c228be27504a2c80d5")

        mock_instance.delete.assert_called_with(
            "https://domain/api/v2/user-blocks/auth0|584ad3c228be27504a2c80d5"
        )



from ..rest import RestClient


class Rules:
    """Rules endpoint implementation.

    Args:
        domain (str): Your Auth0 domain, e.g: 'username.auth0.com'

        token (str): Management API v2 Token

        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)

        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)

        rest_options (RestClientOptions): Pass an instance of
            RestClientOptions to configure additional RestClient
            options, such as rate-limit retries.
            (defaults to None)
    """

    def __init__(
        self,
        domain,
        token,
        telemetry=True,
        timeout=5.0,
        protocol="https",
        rest_options=None,
    ):
        self.domain = domain
        self.protocol = protocol
        self.client = RestClient(
            jwt=token, telemetry=telemetry, timeout=timeout, options=rest_options
        )

    def _url(self, id=None):
        url = f"{self.protocol}://{self.domain}/api/v2/rules"
        if id is not None:
            return f"{url}/{id}"
        return url

    def all(
        self,
        stage="login_success",
        enabled=True,
        fields=None,
        include_fields=True,
        page=None,
        per_page=None,
        include_totals=False,
    ):
        """Retrieves a list of all rules.

        Args:
            stage (str, optional):  Retrieves rules that match the execution stage.
                Defaults to login_success.

            enabled (bool, optional): If provided, retrieves rules that match
                the value, otherwise all rules are retrieved.

            fields (list, optional): A list of fields to include or exclude
                (depending on include_fields) from the result. Leave empty to
                retrieve all fields.

            include_fields (bool, optional): True if the fields specified are
                to be included in the result, False otherwise. Defaults to True.

            page (int, optional): The result's page number (zero based). When not set,
                the default value is up to the server.

            per_page (int, optional): The amount of entries per page. When not set,
                the default value is up to the server.

            include_totals (bool, optional): True if the query summary is
                to be included in the result, False otherwise. Defaults to False.

        See: https://auth0.com/docs/api/management/v2#!/Rules/get_rules
        """

        params = {
            "stage": stage,
            "fields": fields and ",".join(fields) or None,
            "include_fields": str(include_fields).lower(),
            "page": page,
            "per_page": per_page,
            "include_totals": str(include_totals).lower(),
        }

        # since the default is True, this is here to disable the filter
        if enabled is not None:
            params["enabled"] = str(enabled).lower()

        return self.client.get(self._url(), params=params)

    def create(self, body):
        """Creates a new rule.

        Args:
            body (dict): Attributes for the newly created rule.

        See: https://auth0.com/docs/api/v2#!/Rules/post_rules
        """
        return self.client.post(self._url(), data=body)

    def get(self, id, fields=None, include_fields=True):
        """Retrieves a rule by its ID.

        Args:
            id (str): The id of the rule to retrieve.

            fields (list, optional): A list of fields to include or exclude
                (depending on include_fields) from the result. Leave empty to
                retrieve all fields.

            include_fields (bool, optional): True if the fields specified are
                to be included in the result, False otherwise. Defaults to True.

        See: https://auth0.com/docs/api/management/v2#!/Rules/get_rules_by_id
        """
        params = {
            "fields": fields and ",".join(fields) or None,
            "include_fields": str(include_fields).lower(),
        }
        return self.client.get(self._url(id), params=params)

    def delete(self, id):
        """Delete a rule.

        Args:
            id (str): The id of the rule to delete.

        See: https://auth0.com/docs/api/management/v2#!/Rules/delete_rules_by_id
        """
        return self.client.delete(self._url(id))

    def update(self, id, body):
        """Update an existing rule

        Args:
            id (str): The id of the rule to modify.

            body (dict): Attributes to modify.

        See: https://auth0.com/docs/api/v2#!/Rules/patch_rules_by_id
        """
        return self.client.patch(self._url(id), data=body)



import unittest
from unittest import mock

from ...management.emails import Emails


class TestEmails(unittest.TestCase):
    def test_init_with_optionals(self):
        t = Emails(domain="domain", token="jwttoken", telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.options.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get("Auth0-Client", None)
        self.assertEqual(telemetry_header, None)

    @mock.patch("auth0.management.emails.RestClient")
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value

        e = Emails(domain="domain", token="jwttoken")
        e.get()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/emails/provider", args[0])
        self.assertEqual(kwargs["params"], {"fields": None, "include_fields": "true"})

        e.get(fields=["a", "b"], include_fields=False)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/emails/provider", args[0])
        self.assertEqual(kwargs["params"], {"fields": "a,b", "include_fields": "false"})

    @mock.patch("auth0.management.emails.RestClient")
    def test_config(self, mock_rc):
        mock_instance = mock_rc.return_value

        e = Emails(domain="domain", token="jwttoken")
        e.config({"a": "b", "c": "d"})

        args, kwargs = mock_instance.post.call_args

        self.assertEqual("https://domain/api/v2/emails/provider", args[0])
        self.assertEqual(kwargs["data"], {"a": "b", "c": "d"})

    @mock.patch("auth0.management.emails.RestClient")
    def test_update(self, mock_rc):
        mock_instance = mock_rc.return_value

        e = Emails(domain="domain", token="jwttoken")
        e.update({"a": "b", "c": "d"})

        args, kwargs = mock_instance.patch.call_args

        self.assertEqual("https://domain/api/v2/emails/provider", args[0])
        self.assertEqual(kwargs["data"], {"a": "b", "c": "d"})

    @mock.patch("auth0.management.emails.RestClient")
    def test_delete(self, mock_rc):
        mock_instance = mock_rc.return_value

        e = Emails(domain="domain", token="jwttoken")
        e.delete()

        mock_instance.delete.assert_called_with("https://domain/api/v2/emails/provider")



import unittest
from unittest import mock

from ...management.grants import Grants


class TestGrants(unittest.TestCase):
    def test_init_with_optionals(self):
        t = Grants(domain="domain", token="jwttoken", telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.options.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get("Auth0-Client", None)
        self.assertEqual(telemetry_header, None)

    @mock.patch("auth0.management.grants.RestClient")
    def test_get_all(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Grants(domain="domain", token="jwttoken")
        g.all(
            extra_params={"user_id": "an-id", "client_id": "an-id", "audience": "test"}
        )

        args, kwargs = mock_instance.get.call_args

        mock_instance.get.assert_called_with(
            "https://domain/api/v2/grants",
            params={
                "user_id": "an-id",
                "client_id": "an-id",
                "audience": "test",
                "page": None,
                "per_page": None,
                "include_totals": "false",
            },
        )

    @mock.patch("auth0.management.grants.RestClient")
    def test_delete(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Grants(domain="domain", token="jwttoken")
        c.delete("an-id")

        mock_instance.delete.assert_called_with("https://domain/api/v2/grants/an-id")



from ..rest import RestClient


class Connections:
    """Auth0 connection endpoints

    Args:
        domain (str): Your Auth0 domain, e.g: 'username.auth0.com'

        token (str): Management API v2 Token

        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)

        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)

        rest_options (RestClientOptions): Pass an instance of
            RestClientOptions to configure additional RestClient
            options, such as rate-limit retries.
            (defaults to None)
    """

    def __init__(
        self,
        domain,
        token,
        telemetry=True,
        timeout=5.0,
        protocol="https",
        rest_options=None,
    ):
        self.domain = domain
        self.protocol = protocol
        self.client = RestClient(
            jwt=token, telemetry=telemetry, timeout=timeout, options=rest_options
        )

    def _url(self, id=None):
        url = f"{self.protocol}://{self.domain}/api/v2/connections"
        if id is not None:
            return f"{url}/{id}"
        return url

    def all(
        self,
        strategy=None,
        fields=None,
        include_fields=True,
        page=None,
        per_page=None,
        extra_params=None,
    ):
        """Retrieves all connections.

        Args:
           strategy (str, optional): Only retrieve connections of
              this strategy type. (e.g: strategy='amazon')

           fields (list of str, optional): A list of fields to include or
              exclude from the result (depending on include_fields). Leave empty to
              retrieve all fields. By default, all the fields will be retrieved.

           include_fields (bool, optional): True if the fields specified are
              to be included in the result, False otherwise. Defaults to True.

           page (int): The result's page number (zero based). When not set,
              the default value is up to the server.

           per_page (int, optional): The amount of entries per page. When not set,
              the default value is up to the server.

           extra_params (dictionary, optional): The extra parameters to add to
             the request. The fields, include_fields, page and per_page values
             specified as parameters take precedence over the ones defined here.

        See: https://auth0.com/docs/api/management/v2#!/Connections/get_connections

        Returns:
           A list of connection objects.
        """

        params = extra_params or {}
        params["strategy"] = strategy or None
        params["fields"] = fields and ",".join(fields) or None
        params["include_fields"] = str(include_fields).lower()
        params["page"] = page
        params["per_page"] = per_page

        return self.client.get(self._url(), params=params)

    def get(self, id, fields=None, include_fields=True):
        """Retrieve connection by id.

        Args:
           id (str): Id of the connection to get.

           fields (list of str, optional): A list of fields to include or
              exclude from the result (depending on include_fields). Leave empty to
              retrieve all fields. By default, all the fields will be retrieved.

           include_fields (bool, optional): True if the fields specified are
              to be included in the result, False otherwise. Defaults to True.

        See: https://auth0.com/docs/api/management/v2#!/Connections/get_connections_by_id

        Returns:
            A connection object.
        """

        params = {
            "fields": fields and ",".join(fields) or None,
            "include_fields": str(include_fields).lower(),
        }

        return self.client.get(self._url(id), params=params)

    def delete(self, id):
        """Deletes a connection and all its users.

        Args:
           id: Id of the connection to delete.

        See: https://auth0.com/docs/api/management/v2#!/Connections/delete_connections_by_id

        Returns:
           An empty dict.
        """

        return self.client.delete(self._url(id))

    def update(self, id, body):
        """Modifies a connection.

        Args:
           id: Id of the connection.

           body (dict): Specifies which fields are to be modified, and to what values.

        See: https://auth0.com/docs/api/management/v2#!/Connections/patch_connections_by_id

        Returns:
           The modified connection object.
        """

        return self.client.patch(self._url(id), data=body)

    def create(self, body):
        """Creates a new connection.

        Args:
            body (dict): Attributes used to create the connection. Mandatory
                attributes are: 'name' and 'strategy'.

        See: https://auth0.com/docs/api/management/v2#!/Connections/post_connections
        """

        return self.client.post(self._url(), data=body)

    def delete_user_by_email(self, id, email):
        """Deletes a specified connection user by its email.

        Args:
           id (str): The id of the connection (must be a database connection).

           email (str): The email of the user to delete.

        See: https://auth0.com/docs/api/management/v2#!/Connections/delete_users_by_email

        Returns:
            An empty dict.
        """
        return self.client.delete(self._url(id) + "/users", params={"email": email})



import unittest
from unittest import mock

from ...management.blacklists import Blacklists


class TestBlacklists(unittest.TestCase):
    def test_init_with_optionals(self):
        t = Blacklists(
            domain="domain", token="jwttoken", telemetry=False, timeout=(10, 2)
        )
        self.assertEqual(t.client.options.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get("Auth0-Client", None)
        self.assertEqual(telemetry_header, None)

    @mock.patch("auth0.management.blacklists.RestClient")
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value

        t = Blacklists(domain="domain", token="jwttoken")
        t.get(aud="an-id")

        mock_instance.get.assert_called_with(
            "https://domain/api/v2/blacklists/tokens", params={"aud": "an-id"}
        )

    @mock.patch("auth0.management.blacklists.RestClient")
    def test_create(self, mock_rc):
        mock_instance = mock_rc.return_value

        t = Blacklists(domain="domain", token="jwttoken")

        # create without audience
        t.create(jti="the-jti")

        args, kwargs = mock_instance.post.call_args

        self.assertEqual("https://domain/api/v2/blacklists/tokens", args[0])
        self.assertEqual(kwargs["data"], {"jti": "the-jti"})

        # create with audience
        t.create(jti="the-jti", aud="the-aud")

        args, kwargs = mock_instance.post.call_args

        self.assertEqual("https://domain/api/v2/blacklists/tokens", args[0])
        self.assertEqual(kwargs["data"], {"jti": "the-jti", "aud": "the-aud"})



import unittest
from unittest import mock

from ...management.guardian import Guardian


class TestGuardian(unittest.TestCase):
    def test_init_with_optionals(self):
        t = Guardian(
            domain="domain", token="jwttoken", telemetry=False, timeout=(10, 2)
        )
        self.assertEqual(t.client.options.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get("Auth0-Client", None)
        self.assertEqual(telemetry_header, None)

    @mock.patch("auth0.management.guardian.RestClient")
    def test_all_factors(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain="domain", token="jwttoken")
        g.all_factors()

        mock_instance.get.assert_called_with("https://domain/api/v2/guardian/factors")

    @mock.patch("auth0.management.guardian.RestClient")
    def test_update_factor(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain="domain", token="jwttoken")
        g.update_factor("push-notification", {"enabled": True})

        args, kwargs = mock_instance.put.call_args
        self.assertEqual(
            "https://domain/api/v2/guardian/factors/push-notification", args[0]
        )
        self.assertEqual(kwargs["data"], {"enabled": True})

        g.update_factor("sms", {"enabled": False})

        args, kwargs = mock_instance.put.call_args
        self.assertEqual("https://domain/api/v2/guardian/factors/sms", args[0])
        self.assertEqual(kwargs["data"], {"enabled": False})

    @mock.patch("auth0.management.guardian.RestClient")
    def test_update_templates(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain="domain", token="jwttoken")
        g.update_templates(
            {"enrollment_message": "hello", "verification_message": "verified"}
        )

        args, kwargs = mock_instance.put.call_args
        self.assertEqual(
            "https://domain/api/v2/guardian/factors/sms/templates", args[0]
        )
        self.assertEqual(
            kwargs["data"],
            {"enrollment_message": "hello", "verification_message": "verified"},
        )

    @mock.patch("auth0.management.guardian.RestClient")
    def test_get_templates(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain="domain", token="jwttoken")
        g.get_templates()

        mock_instance.get.assert_called_with(
            "https://domain/api/v2/guardian/factors/sms/templates"
        )

    @mock.patch("auth0.management.guardian.RestClient")
    def test_get_enrollment(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain="domain", token="jwttoken")
        g.get_enrollment("some_id")

        mock_instance.get.assert_called_with(
            "https://domain/api/v2/guardian/enrollments/some_id"
        )

    @mock.patch("auth0.management.guardian.RestClient")
    def test_delete_enrollment(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain="domain", token="jwttoken")
        g.delete_enrollment("some_id")

        mock_instance.delete.assert_called_with(
            "https://domain/api/v2/guardian/enrollments/some_id"
        )

    @mock.patch("auth0.management.guardian.RestClient")
    def test_create_enrollment_ticket(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain="domain", token="jwttoken")
        g.create_enrollment_ticket(
            {"user_id": "some_id", "email": "test@test.com", "send_mail": "false"}
        )

        args, kwargs = mock_instance.post.call_args
        self.assertEqual("https://domain/api/v2/guardian/enrollments/ticket", args[0])
        self.assertEqual(
            kwargs["data"],
            {"user_id": "some_id", "email": "test@test.com", "send_mail": "false"},
        )

    @mock.patch("auth0.management.guardian.RestClient")
    def test_get_factor_providers(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain="domain", token="jwttoken")
        g.get_factor_providers("sms", "twilio")

        mock_instance.get.assert_called_with(
            "https://domain/api/v2/guardian/factors/sms/providers/twilio"
        )

    @mock.patch("auth0.management.guardian.RestClient")
    def test_update_factor_providers(self, mock_rc):
        mock_instance = mock_rc.return_value

        g = Guardian(domain="domain", token="jwttoken")
        g.update_factor_providers(
            "sms",
            "twilio",
            {
                "from": "test@test.com",
                "messaging_service_sid": "qwerty",
                "auth_token": "abc.xyz.123",
                "sid": "abc.xyz",
            },
        )

        args, kwargs = mock_instance.put.call_args
        self.assertEqual(
            "https://domain/api/v2/guardian/factors/sms/providers/twilio", args[0]
        )
        self.assertEqual(
            kwargs["data"],
            {
                "from": "test@test.com",
                "messaging_service_sid": "qwerty",
                "auth_token": "abc.xyz.123",
                "sid": "abc.xyz",
            },
        )



import unittest
from unittest import mock

from ...management.actions import Actions


class TestActions(unittest.TestCase):
    def test_init_with_optionals(self):
        t = Actions(domain="domain", token="jwttoken", telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.options.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get("Auth0-Client", None)
        self.assertEqual(telemetry_header, None)

    @mock.patch("auth0.management.actions.RestClient")
    def test_get_actions(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Actions(domain="domain", token="jwttoken")

        c.get_actions()
        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/actions/actions", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "triggerId": None,
                "actionName": None,
                "deployed": None,
                "installed": "false",
                "page": None,
                "per_page": None,
            },
        )

        c.get_actions("trigger-id", "action-name", True, True, 0, 5)
        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/actions/actions", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "triggerId": "trigger-id",
                "actionName": "action-name",
                "deployed": "true",
                "installed": "true",
                "page": 0,
                "per_page": 5,
            },
        )

        c.get_actions("trigger-id", "action-name", False, True, 0, 5)
        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/actions/actions", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "triggerId": "trigger-id",
                "actionName": "action-name",
                "deployed": "false",
                "installed": "true",
                "page": 0,
                "per_page": 5,
            },
        )

    @mock.patch("auth0.management.actions.RestClient")
    def test_create_action(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Actions(domain="domain", token="jwttoken")
        c.create_action({"a": "b", "c": "d"})

        mock_instance.post.assert_called_with(
            "https://domain/api/v2/actions/actions", data={"a": "b", "c": "d"}
        )

    @mock.patch("auth0.management.actions.RestClient")
    def test_update_action(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Actions(domain="domain", token="jwttoken")
        c.update_action("action-id", {"a": "b", "c": "d"})

        args, kwargs = mock_instance.patch.call_args

        self.assertEqual("https://domain/api/v2/actions/actions/action-id", args[0])
        self.assertEqual(kwargs["data"], {"a": "b", "c": "d"})

    @mock.patch("auth0.management.actions.RestClient")
    def test_get_action(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Actions(domain="domain", token="jwttoken")
        c.get_action("action-id")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/actions/actions/action-id", args[0])
        self.assertEqual(kwargs["params"], {})

    @mock.patch("auth0.management.actions.RestClient")
    def test_get_triggers(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Actions(domain="domain", token="jwttoken")
        c.get_triggers()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/actions/triggers", args[0])
        self.assertEqual(kwargs["params"], {})

    @mock.patch("auth0.management.actions.RestClient")
    def test_delete_action(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Actions(domain="domain", token="jwttoken")
        c.delete_action("action-id")

        args, kwargs = mock_instance.delete.call_args

        self.assertEqual("https://domain/api/v2/actions/actions/action-id", args[0])
        self.assertEqual(kwargs["params"], {"force": "false"})

        c.delete_action("action-id", True)

        args, kwargs = mock_instance.delete.call_args

        self.assertEqual("https://domain/api/v2/actions/actions/action-id", args[0])
        self.assertEqual(kwargs["params"], {"force": "true"})

    @mock.patch("auth0.management.actions.RestClient")
    def test_get_execution(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Actions(domain="domain", token="jwttoken")
        c.get_execution("execution-id")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/actions/executions/execution-id", args[0]
        )
        self.assertEqual(kwargs["params"], {})

    @mock.patch("auth0.management.actions.RestClient")
    def test_get_action_versions(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Actions(domain="domain", token="jwttoken")
        c.get_action_versions("action-id")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/actions/actions/action-id/versions", args[0]
        )
        self.assertEqual(kwargs["params"], {"page": None, "per_page": None})

        c.get_action_versions("action-id", 0, 5)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/actions/actions/action-id/versions", args[0]
        )
        self.assertEqual(kwargs["params"], {"page": 0, "per_page": 5})

    @mock.patch("auth0.management.actions.RestClient")
    def test_get_trigger_bindings(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Actions(domain="domain", token="jwttoken")
        c.get_trigger_bindings("trigger-id")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/actions/triggers/trigger-id/bindings", args[0]
        )
        self.assertEqual(kwargs["params"], {"page": None, "per_page": None})

        c.get_trigger_bindings("trigger-id", 0, 5)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/actions/triggers/trigger-id/bindings", args[0]
        )
        self.assertEqual(kwargs["params"], {"page": 0, "per_page": 5})

    @mock.patch("auth0.management.actions.RestClient")
    def test_get_action_version(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Actions(domain="domain", token="jwttoken")
        c.get_action_version("action-id", "version-id")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/actions/actions/action-id/versions/version-id",
            args[0],
        )
        self.assertEqual(kwargs["params"], {})

    @mock.patch("auth0.management.actions.RestClient")
    def test_deploy_action(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Actions(domain="domain", token="jwttoken")
        c.deploy_action("action-id")

        args, kwargs = mock_instance.post.call_args

        self.assertEqual(
            "https://domain/api/v2/actions/actions/action-id/deploy", args[0]
        )

    @mock.patch("auth0.management.actions.RestClient")
    def test_rollback_action(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Actions(domain="domain", token="jwttoken")
        c.rollback_action_version("action-id", "version-id")

        args, kwargs = mock_instance.post.call_args

        self.assertEqual(
            "https://domain/api/v2/actions/actions/action-id/versions/version-id/deploy",
            args[0],
        )
        self.assertEqual(kwargs["data"], {})

    @mock.patch("auth0.management.actions.RestClient")
    def test_update_trigger_bindings(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = Actions(domain="domain", token="jwttoken")
        c.update_trigger_bindings("trigger-id", {"a": "b", "c": "d"})

        args, kwargs = mock_instance.patch.call_args

        self.assertEqual(
            "https://domain/api/v2/actions/triggers/trigger-id/bindings", args[0]
        )
        self.assertEqual(kwargs["data"], {"a": "b", "c": "d"})



import aiohttp

from ..asyncify import asyncify
from .auth0 import modules


class AsyncAuth0:
    """Provides easy access to all endpoint classes

    Args:
        domain (str): Your Auth0 domain, for example 'username.auth0.com'

        token (str): Management API v2 Token

        rest_options (RestClientOptions): Pass an instance of
            RestClientOptions to configure additional RestClient
            options, such as rate-limit retries.
            (defaults to None)
    """

    def __init__(self, domain, token, rest_options=None):
        self._services = []
        for name, cls in modules.items():
            cls = asyncify(cls)
            service = cls(domain=domain, token=token, rest_options=rest_options)
            self._services.append(service)
            setattr(
                self,
                name,
                service,
            )

    def set_session(self, session):
        """Set Client Session to improve performance by reusing session.

        Args:
            session (aiohttp.ClientSession): The client session which should be closed
                manually or within context manager.
        """
        self._session = session
        for service in self._services:
            service.set_session(self._session)

    async def __aenter__(self):
        """Automatically create and set session within context manager."""
        self.set_session(aiohttp.ClientSession())
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """Automatically close session within context manager."""
        await self._session.close()



import unittest
from unittest import mock

from ...management.log_streams import LogStreams


class TestLogStreams(unittest.TestCase):
    def test_init_with_optionals(self):
        t = LogStreams(
            domain="domain", token="jwttoken", telemetry=False, timeout=(10, 2)
        )
        self.assertEqual(t.client.options.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get("Auth0-Client", None)
        self.assertEqual(telemetry_header, None)

    @mock.patch("auth0.management.log_streams.RestClient")
    def test_list(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = LogStreams(domain="domain", token="jwttoken")

        c.list()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/log-streams", args[0])

    @mock.patch("auth0.management.log_streams.RestClient")
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = LogStreams(domain="domain", token="jwttoken")
        c.get("an-id")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/log-streams/an-id", args[0])

    @mock.patch("auth0.management.log_streams.RestClient")
    def test_create(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = LogStreams(domain="domain", token="jwttoken")
        # Sample data belongs to an `http` stream
        log_stream_data = {
            "name": "string",
            "type": "http",
            "sink": {
                "httpEndpoint": "string",
                "httpContentType": "string",
                "httpContentFormat": "JSONLINES|JSONARRAY",
                "httpAuthorization": "string",
            },
        }
        c.create(log_stream_data)

        args, kwargs = mock_instance.post.call_args

        self.assertEqual("https://domain/api/v2/log-streams", args[0])
        self.assertEqual(kwargs["data"], log_stream_data)

    @mock.patch("auth0.management.log_streams.RestClient")
    def test_delete(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = LogStreams(domain="domain", token="jwttoken")
        c.delete("an-id")

        mock_instance.delete.assert_called_with(
            "https://domain/api/v2/log-streams/an-id"
        )

    @mock.patch("auth0.management.log_streams.RestClient")
    def test_update(self, mock_rc):
        mock_instance = mock_rc.return_value
        log_stream_update = {"name": "string"}

        c = LogStreams(domain="domain", token="jwttoken")
        c.update("an-id", log_stream_update)

        args, kwargs = mock_instance.patch.call_args

        self.assertEqual("https://domain/api/v2/log-streams/an-id", args[0])
        self.assertEqual(kwargs["data"], log_stream_update)



import unittest
from unittest import mock

from callee.strings import Glob
from cryptography.hazmat.primitives import asymmetric, serialization

from ...authentication.get_token import GetToken


def get_private_key():
    private_key = asymmetric.rsa.generate_private_key(
        public_exponent=65537,
        key_size=2048,
    )
    return private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=serialization.NoEncryption(),
    )


class TestGetToken(unittest.TestCase):
    @mock.patch("auth0.rest.RestClient.post")
    def test_authorization_code(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="clsec")

        g.authorization_code(
            code="cd",
            grant_type="gt",
            redirect_uri="idt",
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "clsec",
                "code": "cd",
                "grant_type": "gt",
                "redirect_uri": "idt",
            },
        )

    @mock.patch("auth0.rest.RestClient.post")
    def test_authorization_code_with_client_assertion(self, mock_post):
        g = GetToken(
            "my.domain.com", "cid", client_assertion_signing_key=get_private_key()
        )

        g.authorization_code(code="cd", grant_type="gt", redirect_uri="idt")

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_assertion": Glob("*.*.*"),
                "client_assertion_type": "urn:ietf:params:oauth:client-assertion-type:jwt-bearer",
                "code": "cd",
                "grant_type": "gt",
                "redirect_uri": "idt",
            },
        )

    @mock.patch("auth0.rest.RestClient.post")
    def test_authorization_code_pkce(self, mock_post):
        g = GetToken("my.domain.com", "cid")

        g.authorization_code_pkce(
            code_verifier="cdver",
            code="cd",
            grant_type="gt",
            redirect_uri="idt",
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "code_verifier": "cdver",
                "code": "cd",
                "grant_type": "gt",
                "redirect_uri": "idt",
            },
        )

    @mock.patch("auth0.rest.RestClient.post")
    def test_client_credentials(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="clsec")

        g.client_credentials(audience="aud", grant_type="gt")

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "clsec",
                "audience": "aud",
                "grant_type": "gt",
            },
        )

    @mock.patch("auth0.rest.RestClient.post")
    def test_client_credentials_with_client_assertion(self, mock_post):
        g = GetToken(
            "my.domain.com", "cid", client_assertion_signing_key=get_private_key()
        )

        g.client_credentials("aud", grant_type="gt")

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_assertion": Glob("*.*.*"),
                "client_assertion_type": "urn:ietf:params:oauth:client-assertion-type:jwt-bearer",
                "audience": "aud",
                "grant_type": "gt",
            },
        )

    @mock.patch("auth0.rest.RestClient.post")
    def test_login(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="clsec")

        g.login(
            username="usrnm",
            password="pswd",
            scope="http://test.com/api",
            realm="rlm",
            audience="aud",
            grant_type="gt",
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "clsec",
                "username": "usrnm",
                "password": "pswd",
                "scope": "http://test.com/api",
                "realm": "rlm",
                "audience": "aud",
                "grant_type": "gt",
            },
        )

    @mock.patch("auth0.rest.RestClient.post")
    def test_refresh_token(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="clsec")

        g.refresh_token(
            refresh_token="rt",
            grant_type="gt",
            scope="s",
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "clsec",
                "refresh_token": "rt",
                "grant_type": "gt",
                "scope": "s",
            },
        )

    @mock.patch("auth0.rest.RestClient.post")
    def test_passwordless_login_with_sms(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="csec")

        g.passwordless_login(
            username="123456",
            otp="abcd",
            realm="sms",
            audience="aud",
            scope="openid",
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "csec",
                "realm": "sms",
                "grant_type": "http://auth0.com/oauth/grant-type/passwordless/otp",
                "username": "123456",
                "otp": "abcd",
                "audience": "aud",
                "scope": "openid",
            },
        )

    @mock.patch("auth0.rest.RestClient.post")
    def test_passwordless_login_with_email(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="csec")

        g.passwordless_login(
            username="a@b.c",
            otp="abcd",
            realm="email",
            audience="aud",
            scope="openid",
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "csec",
                "realm": "email",
                "grant_type": "http://auth0.com/oauth/grant-type/passwordless/otp",
                "username": "a@b.c",
                "otp": "abcd",
                "audience": "aud",
                "scope": "openid",
            },
        )



from ..rest import RestClient


class Branding:
    """Auth0 Branding endpoints

    Args:
        domain (str): Your Auth0 domain, e.g: 'username.auth0.com'

        token (str): Management API v2 Token

        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)

        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)

        rest_options (RestClientOptions): Pass an instance of
            RestClientOptions to configure additional RestClient
            options, such as rate-limit retries.
            (defaults to None)
    """

    def __init__(
        self,
        domain,
        token,
        telemetry=True,
        timeout=5.0,
        protocol="https",
        rest_options=None,
    ):
        self.domain = domain
        self.protocol = protocol
        self.client = RestClient(
            jwt=token, telemetry=telemetry, timeout=timeout, options=rest_options
        )

    def _url(self, *args):
        url = f"{self.protocol}://{self.domain}/api/v2/branding"
        for p in args:
            if p is not None:
                url = f"{url}/{p}"
        return url

    def get(self, aud=None):
        """Retrieve branding settings. Requires "read:branding" scope.

        See: https://auth0.com/docs/api/management/v2#!/Branding/get_branding
        """

        return self.client.get(self._url())

    def update(self, body):
        """Update branding settings. Requires "update:branding" scope.

        Args:
            body (dict): Attributes for the updated trigger binding.

        See: https://auth0.com/docs/api/management/v2#!/Branding/patch_branding
        """

        return self.client.patch(self._url(), data=body)

    def get_template_universal_login(self):
        """Get template for New Universal Login Experience. Requires "read:branding" scope.

        See: https://auth0.com/docs/api/management/v2#!/Branding/get_universal_login
        """

        return self.client.get(self._url("templates", "universal-login"))

    def delete_template_universal_login(self):
        """Delete template for New Universal Login Experience. Requires "delete:branding" scope.

        See: https://auth0.com/docs/api/management/v2#!/Branding/delete_universal_login
        """

        return self.client.delete(self._url("templates", "universal-login"))

    def update_template_universal_login(self, body):
        """Update template for New Universal Login Experience. Requires "update:branding" scope.

        Args:
            body (str): Complete HTML content to assign to the template. See linked API documentation for example.

        See: https://auth0.com/docs/api/management/v2#!/Branding/put_universal_login
        """

        return self.client.put(
            self._url("templates", "universal-login"),
            body={"template": body},
        )

    def get_default_branding_theme(self):
        """Retrieve default branding theme.

        See: https://auth0.com/docs/api/management/v2#!/Branding/get_default_branding_theme
        """

        return self.client.get(self._url("themes", "default"))

    def get_branding_theme(self, theme_id):
        """Retrieve branding theme.

        Args:
            theme_id (str): The theme_id to retrieve branding theme for.

        See: https://auth0.com/docs/api/management/v2#!/Branding/get_branding_theme
        """

        return self.client.get(self._url("themes", theme_id))

    def delete_branding_theme(self, theme_id):
        """Delete branding theme.

        Args:
            theme_id (str): The theme_id to delete branding theme for.

        See: https://auth0.com/docs/api/management/v2#!/Branding/delete_branding_theme
        """

        return self.client.delete(self._url("themes", theme_id))

    def update_branding_theme(self, theme_id, body):
        """Update branding theme.

        Args:
            theme_id (str): The theme_id to update branding theme for.
            body (dict): The attributes to set on the theme.

        See: https://auth0.com/docs/api/management/v2#!/Branding/patch_branding_theme
        """

        return self.client.patch(self._url("themes", theme_id), data=body)

    def create_branding_theme(self, body):
        """Create branding theme.

        Args:
            body (dict): The attributes to set on the theme.

        See: https://auth0.com/docs/api/management/v2#!/Branding/post_branding_theme
        """

        return self.client.post(self._url("themes"), data=body)



import unittest
from unittest import mock

from ...management.prompts import Prompts


class TestPrompts(unittest.TestCase):
    def test_init_with_optionals(self):
        t = Prompts(domain="domain", token="jwttoken", telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.options.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get("Auth0-Client", None)
        self.assertEqual(telemetry_header, None)

    @mock.patch("auth0.management.prompts.RestClient")
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value

        p = Prompts(domain="domain", token="jwttoken")
        p.get()

        args, _ = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/prompts", args[0])

    @mock.patch("auth0.management.prompts.RestClient")
    def test_update(self, mock_rc):
        mock_instance = mock_rc.return_value

        p = Prompts(domain="domain", token="jwttoken")
        p.update({"a": "b", "c": "d"})

        args, kwargs = mock_instance.patch.call_args

        self.assertEqual("https://domain/api/v2/prompts", args[0])
        self.assertEqual(kwargs["data"], {"a": "b", "c": "d"})

    @mock.patch("auth0.management.prompts.RestClient")
    def test_get_custom_text(self, mock_rc):
        mock_instance = mock_rc.return_value

        p = Prompts(domain="domain", token="jwttoken")
        p.get_custom_text("some-prompt", "some-language")

        args, _ = mock_instance.get.call_args

        self.assertEqual(
            "https://domain/api/v2/prompts/some-prompt/custom-text/some-language",
            args[0],
        )

    @mock.patch("auth0.management.prompts.RestClient")
    def test_update_custom_text(self, mock_rc):
        mock_instance = mock_rc.return_value

        p = Prompts(domain="domain", token="jwttoken")
        p.update_custom_text("some-prompt", "some-language", {"a": "b", "c": "d"})

        args, kwargs = mock_instance.put.call_args

        self.assertEqual(
            "https://domain/api/v2/prompts/some-prompt/custom-text/some-language",
            args[0],
        )
        self.assertEqual(kwargs["data"], {"a": "b", "c": "d"})



import unittest
from unittest import mock

from ...management.device_credentials import DeviceCredentials


class TestDeviceCredentials(unittest.TestCase):
    def test_init_with_optionals(self):
        t = DeviceCredentials(
            domain="domain", token="jwttoken", telemetry=False, timeout=(10, 2)
        )
        self.assertEqual(t.client.options.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get("Auth0-Client", None)
        self.assertEqual(telemetry_header, None)

    @mock.patch("auth0.management.device_credentials.RestClient")
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = DeviceCredentials(domain="domain", token="jwttoken")
        c.get(user_id="uid", client_id="cid", type="type", page=0, per_page=20)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/device-credentials", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "fields": None,
                "include_fields": "true",
                "user_id": "uid",
                "client_id": "cid",
                "type": "type",
                "page": 0,
                "per_page": 20,
                "include_totals": "false",
            },
        )

        c.get(
            user_id="uid",
            client_id="cid",
            type="type",
            page=5,
            per_page=50,
            include_totals=True,
        )

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/device-credentials", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "fields": None,
                "include_fields": "true",
                "user_id": "uid",
                "client_id": "cid",
                "type": "type",
                "page": 5,
                "per_page": 50,
                "include_totals": "true",
            },
        )

    @mock.patch("auth0.management.device_credentials.RestClient")
    def test_create(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = DeviceCredentials(domain="domain", token="jwttoken")
        c.create({"a": "b", "c": "d"})

        args, kwargs = mock_instance.post.call_args

        self.assertEqual("https://domain/api/v2/device-credentials", args[0])
        self.assertEqual(kwargs["data"], {"a": "b", "c": "d"})

    @mock.patch("auth0.management.device_credentials.RestClient")
    def test_delete(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = DeviceCredentials(domain="domain", token="jwttoken")
        c.delete("an-id")

        mock_instance.delete.assert_called_with(
            "https://domain/api/v2/device-credentials/an-id",
        )



import unittest
from unittest import mock

from ...management.users import Users


class TestUsers(unittest.TestCase):
    def test_init_with_optionals(self):
        t = Users(domain="domain", token="jwttoken", telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.options.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get("Auth0-Client", None)
        self.assertEqual(telemetry_header, None)

    @mock.patch("auth0.management.users.RestClient")
    def test_list(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.list()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/users", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "per_page": 25,
                "page": 0,
                "include_totals": "true",
                "sort": None,
                "connection": None,
                "fields": None,
                "include_fields": "true",
                "q": None,
                "search_engine": None,
            },
        )

        u.list(
            page=1,
            per_page=50,
            sort="s",
            connection="con",
            q="q",
            search_engine="se",
            include_totals=False,
            fields=["a", "b"],
            include_fields=False,
        )

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/users", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "per_page": 50,
                "page": 1,
                "include_totals": "false",
                "sort": "s",
                "connection": "con",
                "fields": "a,b",
                "include_fields": "false",
                "q": "q",
                "search_engine": "se",
            },
        )

    @mock.patch("auth0.management.users.RestClient")
    def test_create(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.create({"a": "b", "c": "d"})

        args, kwargs = mock_instance.post.call_args

        self.assertEqual("https://domain/api/v2/users", args[0])
        self.assertEqual(kwargs["data"], {"a": "b", "c": "d"})

    @mock.patch("auth0.management.users.RestClient")
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.get("an-id")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/users/an-id", args[0])
        self.assertEqual(kwargs["params"], {"fields": None, "include_fields": "true"})

        u.get("an-id", fields=["a", "b"], include_fields=False)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/users/an-id", args[0])
        self.assertEqual(kwargs["params"], {"fields": "a,b", "include_fields": "false"})

    @mock.patch("auth0.management.users.RestClient")
    def test_delete(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.delete("an-id")

        mock_instance.delete.assert_called_with("https://domain/api/v2/users/an-id")

    @mock.patch("auth0.management.users.RestClient")
    def test_update(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.update("an-id", {"a": "b", "c": "d"})

        args, kwargs = mock_instance.patch.call_args

        self.assertEqual("https://domain/api/v2/users/an-id", args[0])
        self.assertEqual(kwargs["data"], {"a": "b", "c": "d"})

    @mock.patch("auth0.management.users.RestClient")
    def test_list_organizations(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.list_organizations("an-id")

        args, kwargs = mock_instance.get.call_args
        self.assertEqual("https://domain/api/v2/users/an-id/organizations", args[0])
        self.assertEqual(
            kwargs["params"], {"per_page": 25, "page": 0, "include_totals": "true"}
        )

        u.list_organizations(id="an-id", page=1, per_page=50, include_totals=False)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/users/an-id/organizations", args[0])
        self.assertEqual(
            kwargs["params"], {"per_page": 50, "page": 1, "include_totals": "false"}
        )

    @mock.patch("auth0.management.users.RestClient")
    def test_list_roles(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.list_roles("an-id")

        args, kwargs = mock_instance.get.call_args
        self.assertEqual("https://domain/api/v2/users/an-id/roles", args[0])
        self.assertEqual(
            kwargs["params"], {"per_page": 25, "page": 0, "include_totals": "true"}
        )

        u.list_roles(id="an-id", page=1, per_page=50, include_totals=False)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/users/an-id/roles", args[0])
        self.assertEqual(
            kwargs["params"], {"per_page": 50, "page": 1, "include_totals": "false"}
        )

    @mock.patch("auth0.management.users.RestClient")
    def test_remove_roles(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.remove_roles("an-id", ["a", "b"])

        args, kwargs = mock_instance.delete.call_args

        self.assertEqual("https://domain/api/v2/users/an-id/roles", args[0])
        self.assertEqual(kwargs["data"], {"roles": ["a", "b"]})

    @mock.patch("auth0.management.users.RestClient")
    def test_add_roles(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.add_roles("an-id", ["a", "b"])

        args, kwargs = mock_instance.post.call_args

        self.assertEqual("https://domain/api/v2/users/an-id/roles", args[0])
        self.assertEqual(kwargs["data"], {"roles": ["a", "b"]})

    @mock.patch("auth0.management.users.RestClient")
    def test_list_permissions(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.list_permissions("an-id")

        args, kwargs = mock_instance.get.call_args
        self.assertEqual("https://domain/api/v2/users/an-id/permissions", args[0])
        self.assertEqual(
            kwargs["params"], {"per_page": 25, "page": 0, "include_totals": "true"}
        )

        u.list_permissions(id="an-id", page=1, per_page=50, include_totals=False)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/users/an-id/permissions", args[0])
        self.assertEqual(
            kwargs["params"], {"per_page": 50, "page": 1, "include_totals": "false"}
        )

    @mock.patch("auth0.management.users.RestClient")
    def test_remove_permissions(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.remove_permissions("an-id", ["a", "b"])

        args, kwargs = mock_instance.delete.call_args

        self.assertEqual("https://domain/api/v2/users/an-id/permissions", args[0])
        self.assertEqual(kwargs["data"], {"permissions": ["a", "b"]})

    @mock.patch("auth0.management.users.RestClient")
    def test_add_permissions(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.add_permissions("an-id", ["a", "b"])

        args, kwargs = mock_instance.post.call_args

        self.assertEqual("https://domain/api/v2/users/an-id/permissions", args[0])
        self.assertEqual(kwargs["data"], {"permissions": ["a", "b"]})

    @mock.patch("auth0.management.users.RestClient")
    def test_delete_multifactor(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.delete_multifactor("an-id", "provider")

        mock_instance.delete.assert_called_with(
            "https://domain/api/v2/users/an-id/multifactor/provider"
        )

    @mock.patch("auth0.management.users.RestClient")
    def test_delete_authenticators(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.delete_authenticators("an-id")

        mock_instance.delete.assert_called_with(
            "https://domain/api/v2/users/an-id/authenticators"
        )

    @mock.patch("auth0.management.users.RestClient")
    def test_unlink_user_account(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.unlink_user_account("an-id", "provider", "user-id")

        mock_instance.delete.assert_called_with(
            "https://domain/api/v2/users/an-id/identities/provider/user-id"
        )

    @mock.patch("auth0.management.users.RestClient")
    def test_link_user_account(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.link_user_account("user-id", {"a": "b", "c": "d"})

        args, kwargs = mock_instance.post.call_args

        self.assertEqual("https://domain/api/v2/users/user-id/identities", args[0])
        self.assertEqual(kwargs["data"], {"a": "b", "c": "d"})

    @mock.patch("auth0.management.users.RestClient")
    def test_regenerate_recovery_code(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.regenerate_recovery_code("user-id")

        mock_instance.post.assert_called_with(
            "https://domain/api/v2/users/user-id/recovery-code-regeneration"
        )

    @mock.patch("auth0.management.users.RestClient")
    def test_get_guardian_enrollments(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.get_guardian_enrollments("user-id")

        mock_instance.get.assert_called_with(
            "https://domain/api/v2/users/user-id/enrollments"
        )

    @mock.patch("auth0.management.users.RestClient")
    def test_get_log_events(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.get_log_events("used_id")

        args, kwargs = mock_instance.get.call_args
        self.assertEqual("https://domain/api/v2/users/used_id/logs", args[0])
        self.assertEqual(kwargs["params"]["page"], 0)
        self.assertEqual(kwargs["params"]["per_page"], 50)
        self.assertIsNone(kwargs["params"]["sort"])
        self.assertEqual(kwargs["params"]["include_totals"], "false")

    @mock.patch("auth0.management.users.RestClient")
    def test_invalidate_remembered_browsers(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.invalidate_remembered_browsers("user-id")

        args, kwargs = mock_instance.post.call_args
        self.assertEqual(
            "https://domain/api/v2/users/user-id/multifactor/actions/invalidate-remember-browser",
            args[0],
        )

    @mock.patch("auth0.management.users.RestClient")
    def test_get_authentication_methods(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.get_authentication_methods("user_id")

        mock_instance.get.assert_called_with(
            "https://domain/api/v2/users/user_id/authentication-methods"
        )

    @mock.patch("auth0.management.users.RestClient")
    def test_get_authentication_method_by_id(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.get_authentication_method_by_id("user_id", "authentication_method_id")

        mock_instance.get.assert_called_with(
            "https://domain/api/v2/users/user_id/authentication-methods/authentication_method_id"
        )

    @mock.patch("auth0.management.users.RestClient")
    def test_create_authentication_method(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.create_authentication_method("user_id", {})

        mock_instance.post.assert_called_with(
            "https://domain/api/v2/users/user_id/authentication-methods", data={}
        )

    @mock.patch("auth0.management.users.RestClient")
    def test_update_authentication_methods(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.update_authentication_methods("user_id", {})

        mock_instance.put.assert_called_with(
            "https://domain/api/v2/users/user_id/authentication-methods", data={}
        )

    @mock.patch("auth0.management.users.RestClient")
    def test_update_authentication_method_by_id(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.update_authentication_method_by_id("user_id", "authentication_method_id", {})

        mock_instance.patch.assert_called_with(
            "https://domain/api/v2/users/user_id/authentication-methods/authentication_method_id",
            data={},
        )

    @mock.patch("auth0.management.users.RestClient")
    def test_delete_authentication_methods(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.delete_authentication_methods("user_id")

        mock_instance.delete.assert_called_with(
            "https://domain/api/v2/users/user_id/authentication-methods"
        )

    @mock.patch("auth0.management.users.RestClient")
    def test_delete_authentication_method_by_id(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Users(domain="domain", token="jwttoken")
        u.delete_authentication_method_by_id("user_id", "authentication_method_id")

        mock_instance.delete.assert_called_with(
            "https://domain/api/v2/users/user_id/authentication-methods/authentication_method_id"
        )



from ..rest import RestClient


class ClientCredentials:
    """Auth0 client credentials endpoints.

    Args:
        domain (str): Your Auth0 domain, for example: 'my-domain.us.auth0.com'

        token (str): Management API v2 Token

        telemetry (bool, optional): Enable or disable telemetry
            (defaults to True)

        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)

        rest_options (RestClientOptions): Pass an instance of
            RestClientOptions to configure additional RestClient
            options, such as rate-limit retries.
            (defaults to None)
    """

    def __init__(
        self,
        domain,
        token,
        telemetry=True,
        timeout=5.0,
        protocol="https",
        rest_options=None,
    ):
        self.domain = domain
        self.protocol = protocol
        self.client = RestClient(
            jwt=token, telemetry=telemetry, timeout=timeout, options=rest_options
        )

    def _url(self, client_id, id=None):
        url = "{}://{}/api/v2/clients/{}/credentials".format(
            self.protocol, self.domain, client_id
        )
        if id is not None:
            return f"{url}/{id}"
        return url

    def all(self, client_id):
        """Get a list of credentials associated with a client.

        Args:
            client_id (string): The id of a client that owns the credentials.

        See: https://auth0.com/docs/api/management/v2#!/Client_Credentials/get_client_credentials
        """
        return self.client.get(self._url(client_id))

    def get(self, client_id, id):
        """Retrieve a specified client credential.

        Args:
            client_id (string): The id of a client that owns the credential.

            id (string): The id of the credential.

        See: https://auth0.com/docs/api/management/v2#!/Client_Credentials/get_client_credentials_by_id
        """
        return self.client.get(self._url(client_id, id))

    def create(self, client_id, body):
        """Create a credential on a client.

        Args:
            client_id (string): The id of a client to create the credential for.

        See: https://auth0.com/docs/api/management/v2#!/Client_Credentials/post_client_credentials
        """
        return self.client.post(self._url(client_id), data=body)

    def delete(self, client_id, id):
        """Delete a client's credential.

        Args:
           id (str): The id of credential to delete.

        See: https://auth0.com/docs/api/management/v2#!/Client_Credentials/delete_client_credentials_by_id
        """

        return self.client.delete(self._url(client_id, id))



import unittest
from unittest import mock

from ...management.tenants import Tenants


class TestTenants(unittest.TestCase):
    def test_init_with_optionals(self):
        t = Tenants(domain="domain", token="jwttoken", telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.options.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get("Auth0-Client", None)
        self.assertEqual(telemetry_header, None)

    @mock.patch("auth0.management.tenants.RestClient")
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value
        mock_instance.get.return_value = {}

        t = Tenants(domain="domain", token="jwttoken")
        t.get()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/tenants/settings", args[0])
        self.assertEqual(kwargs["params"], {"fields": None, "include_fields": "true"})

        t.get(fields=["a", "b"], include_fields=False)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/tenants/settings", args[0])
        self.assertEqual(kwargs["params"], {"fields": "a,b", "include_fields": "false"})

        t.get(fields=["a", "b"], include_fields=True)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/tenants/settings", args[0])
        self.assertEqual(kwargs["params"], {"fields": "a,b", "include_fields": "true"})

    @mock.patch("auth0.management.tenants.RestClient")
    def test_update(self, mock_rc):
        mock_instance = mock_rc.return_value
        mock_instance.patch.return_value = {}

        t = Tenants(domain="domain", token="jwttoken")
        t.update({"a": "b", "c": "d"})

        mock_instance.patch.assert_called_with(
            "https://domain/api/v2/tenants/settings", data={"a": "b", "c": "d"}
        )



import unittest
from unittest import mock

from ...management.branding import Branding


class TestBranding(unittest.TestCase):
    def test_init_with_optionals(self):
        branding = Branding(
            domain="domain", token="jwttoken", telemetry=False, timeout=(10, 2)
        )
        self.assertEqual(branding.client.options.timeout, (10, 2))

        telemetry = branding.client.base_headers.get("Auth0-Client", None)
        self.assertEqual(telemetry, None)

    @mock.patch("auth0.management.branding.RestClient")
    def test_get(self, mock_rc):
        api = mock_rc.return_value

        branding = Branding(domain="domain", token="jwttoken")
        branding.get()

        api.get.assert_called_with(
            "https://domain/api/v2/branding",
        )

    @mock.patch("auth0.management.branding.RestClient")
    def test_update(self, mock_rc):
        api = mock_rc.return_value
        api.patch.return_value = {}

        branding = Branding(domain="domain", token="jwttoken")
        branding.update({"a": "b", "c": "d"})

        api.patch.assert_called_with(
            "https://domain/api/v2/branding", data={"a": "b", "c": "d"}
        )

    @mock.patch("auth0.management.branding.RestClient")
    def test_get_template_universal_login(self, mock_rc):
        api = mock_rc.return_value

        branding = Branding(domain="domain", token="jwttoken")
        branding.get_template_universal_login()

        api.get.assert_called_with(
            "https://domain/api/v2/branding/templates/universal-login",
        )

    @mock.patch("auth0.management.branding.RestClient")
    def test_delete_template_universal_login(self, mock_rc):
        api = mock_rc.return_value

        branding = Branding(domain="domain", token="jwttoken")
        branding.delete_template_universal_login()

        api.delete.assert_called_with(
            "https://domain/api/v2/branding/templates/universal-login",
        )

    @mock.patch("auth0.management.branding.RestClient")
    def test_update_template_universal_login(self, mock_rc):
        api = mock_rc.return_value
        api.put.return_value = {}

        branding = Branding(domain="domain", token="jwttoken")
        branding.update_template_universal_login({"a": "b", "c": "d"})

        api.put.assert_called_with(
            "https://domain/api/v2/branding/templates/universal-login",
            body={"template": {"a": "b", "c": "d"}},
        )

    @mock.patch("auth0.management.branding.RestClient")
    def test_get_default_branding_theme(self, mock_rc):
        api = mock_rc.return_value
        api.get.return_value = {}

        branding = Branding(domain="domain", token="jwttoken")
        branding.get_default_branding_theme()

        api.get.assert_called_with(
            "https://domain/api/v2/branding/themes/default",
        )

    @mock.patch("auth0.management.branding.RestClient")
    def test_get_branding_theme(self, mock_rc):
        api = mock_rc.return_value
        api.get.return_value = {}

        branding = Branding(domain="domain", token="jwttoken")
        branding.get_branding_theme("theme_id")

        api.get.assert_called_with(
            "https://domain/api/v2/branding/themes/theme_id",
        )

    @mock.patch("auth0.management.branding.RestClient")
    def test_delete_branding_theme(self, mock_rc):
        api = mock_rc.return_value
        api.delete.return_value = {}

        branding = Branding(domain="domain", token="jwttoken")
        branding.delete_branding_theme("theme_id")

        api.delete.assert_called_with(
            "https://domain/api/v2/branding/themes/theme_id",
        )

    @mock.patch("auth0.management.branding.RestClient")
    def test_update_branding_theme(self, mock_rc):
        api = mock_rc.return_value
        api.patch.return_value = {}

        branding = Branding(domain="domain", token="jwttoken")
        branding.update_branding_theme("theme_id", {})

        api.patch.assert_called_with(
            "https://domain/api/v2/branding/themes/theme_id",
            data={},
        )

    @mock.patch("auth0.management.branding.RestClient")
    def test_create_branding_theme(self, mock_rc):
        api = mock_rc.return_value
        api.post.return_value = {}

        branding = Branding(domain="domain", token="jwttoken")
        branding.create_branding_theme({})

        api.post.assert_called_with(
            "https://domain/api/v2/branding/themes",
            data={},
        )



from ..rest import RestClient


class Prompts:
    """Auth0 prompts endpoints

    Args:
        domain (str): Your Auth0 domain, e.g: 'username.auth0.com'

        token (str): Management API v2 Token

        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)

        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)

        rest_options (RestClientOptions): Pass an instance of
            RestClientOptions to configure additional RestClient
            options, such as rate-limit retries.
            (defaults to None)
    """

    def __init__(
        self,
        domain,
        token,
        telemetry=True,
        timeout=5.0,
        protocol="https",
        rest_options=None,
    ):
        self.domain = domain
        self.protocol = protocol
        self.client = RestClient(
            jwt=token, telemetry=telemetry, timeout=timeout, options=rest_options
        )

    def _url(self, prompt=None, language=None):
        url = f"{self.protocol}://{self.domain}/api/v2/prompts"
        if prompt is not None and language is not None:
            return f"{url}/{prompt}/custom-text/{language}"
        return url

    def get(self):
        """Retrieves prompts settings.

        See: https://auth0.com/docs/api/management/v2#!/Prompts/get_prompts
        """

        return self.client.get(self._url())

    def update(self, body):
        """Updates prompts settings.

        See: https://auth0.com/docs/api/management/v2#!/Prompts/patch_prompts
        """

        return self.client.patch(self._url(), data=body)

    def get_custom_text(self, prompt, language):
        """Retrieves custom text for a prompt in a specific language.

        See: https://auth0.com/docs/api/management/v2#!/Prompts/get_custom_text_by_language
        """

        return self.client.get(self._url(prompt, language))

    def update_custom_text(self, prompt, language, body):
        """Updates custom text for a prompt in a specific language.

        See: https://auth0.com/docs/api/management/v2#!/Prompts/put_custom_text_by_language
        """

        return self.client.put(self._url(prompt, language), data=body)



import unittest
from unittest import mock

from ...management.roles import Roles


class TestRoles(unittest.TestCase):
    def test_init_with_optionals(self):
        t = Roles(domain="domain", token="jwttoken", telemetry=False, timeout=(10, 2))
        self.assertEqual(t.client.options.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get("Auth0-Client", None)
        self.assertEqual(telemetry_header, None)

    @mock.patch("auth0.management.roles.RestClient")
    def test_list(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Roles(domain="domain", token="jwttoken")
        u.list()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/roles", args[0])
        self.assertEqual(
            kwargs["params"],
            {"per_page": 25, "page": 0, "include_totals": "true", "name_filter": None},
        )

        u.list(page=1, per_page=50, include_totals=False, name_filter="little-role")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/roles", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "per_page": 50,
                "page": 1,
                "include_totals": "false",
                "name_filter": "little-role",
            },
        )

    @mock.patch("auth0.management.roles.RestClient")
    def test_create(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Roles(domain="domain", token="jwttoken")
        u.create({"a": "b", "c": "d"})

        args, kwargs = mock_instance.post.call_args

        self.assertEqual("https://domain/api/v2/roles", args[0])
        self.assertEqual(kwargs["data"], {"a": "b", "c": "d"})

    @mock.patch("auth0.management.roles.RestClient")
    def test_get(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Roles(domain="domain", token="jwttoken")
        u.get("an-id")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/roles/an-id", args[0])

    @mock.patch("auth0.management.roles.RestClient")
    def test_delete(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Roles(domain="domain", token="jwttoken")
        u.delete("an-id")

        mock_instance.delete.assert_called_with("https://domain/api/v2/roles/an-id")

    @mock.patch("auth0.management.roles.RestClient")
    def test_update(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Roles(domain="domain", token="jwttoken")
        u.update("an-id", {"a": "b", "c": "d"})

        args, kwargs = mock_instance.patch.call_args

        self.assertEqual("https://domain/api/v2/roles/an-id", args[0])
        self.assertEqual(kwargs["data"], {"a": "b", "c": "d"})

    @mock.patch("auth0.management.roles.RestClient")
    def test_list_users(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Roles(domain="domain", token="jwttoken")
        u.list_users("an-id")

        args, kwargs = mock_instance.get.call_args
        self.assertEqual("https://domain/api/v2/roles/an-id/users", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "per_page": 25,
                "page": 0,
                "include_totals": "true",
                "from": None,
                "take": None,
            },
        )

        u.list_users(id="an-id", page=1, per_page=50, include_totals=False)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/roles/an-id/users", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "per_page": 50,
                "page": 1,
                "include_totals": "false",
                "from": None,
                "take": None,
            },
        )

        u.list_users(id="an-id", from_param=8675309, take=75)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/roles/an-id/users", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "from": 8675309,
                "take": 75,
                "per_page": 25,
                "page": 0,
                "include_totals": "true",
            },
        )

    @mock.patch("auth0.management.roles.RestClient")
    def test_add_users(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Roles(domain="domain", token="jwttoken")
        u.add_users("an-id", ["a", "b"])

        args, kwargs = mock_instance.post.call_args

        self.assertEqual("https://domain/api/v2/roles/an-id/users", args[0])
        self.assertEqual(kwargs["data"], {"users": ["a", "b"]})

    @mock.patch("auth0.management.roles.RestClient")
    def test_list_permissions(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Roles(domain="domain", token="jwttoken")
        u.list_permissions("an-id")

        args, kwargs = mock_instance.get.call_args
        self.assertEqual("https://domain/api/v2/roles/an-id/permissions", args[0])
        self.assertEqual(
            kwargs["params"], {"per_page": 25, "page": 0, "include_totals": "true"}
        )

        u.list_permissions(id="an-id", page=1, per_page=50, include_totals=False)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/roles/an-id/permissions", args[0])
        self.assertEqual(
            kwargs["params"], {"per_page": 50, "page": 1, "include_totals": "false"}
        )

    @mock.patch("auth0.management.roles.RestClient")
    def test_remove_permissions(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Roles(domain="domain", token="jwttoken")
        u.remove_permissions("an-id", ["a", "b"])

        args, kwargs = mock_instance.delete.call_args

        self.assertEqual("https://domain/api/v2/roles/an-id/permissions", args[0])
        self.assertEqual(kwargs["data"], {"permissions": ["a", "b"]})

    @mock.patch("auth0.management.roles.RestClient")
    def test_add_permissions(self, mock_rc):
        mock_instance = mock_rc.return_value

        u = Roles(domain="domain", token="jwttoken")
        u.add_permissions("an-id", ["a", "b"])

        args, kwargs = mock_instance.post.call_args

        self.assertEqual("https://domain/api/v2/roles/an-id/permissions", args[0])
        self.assertEqual(kwargs["data"], {"permissions": ["a", "b"]})



import unittest
from unittest import mock

from ...management.client_grants import ClientGrants


class TestClientGrants(unittest.TestCase):
    def test_init_with_optionals(self):
        t = ClientGrants(
            domain="domain", token="jwttoken", telemetry=False, timeout=(10, 2)
        )
        self.assertEqual(t.client.options.timeout, (10, 2))
        telemetry_header = t.client.base_headers.get("Auth0-Client", None)
        self.assertEqual(telemetry_header, None)

    @mock.patch("auth0.management.client_grants.RestClient")
    def test_all(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = ClientGrants(domain="domain", token="jwttoken")

        # With default params
        c.all()

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/client-grants", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "audience": None,
                "page": None,
                "per_page": None,
                "include_totals": "false",
                "client_id": None,
            },
        )

        # With audience
        c.all(audience="http://domain.auth0.com/api/v2/")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/client-grants", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "audience": "http://domain.auth0.com/api/v2/",
                "page": None,
                "per_page": None,
                "include_totals": "false",
                "client_id": None,
            },
        )

        # With pagination params
        c.all(per_page=23, page=7, include_totals=True)

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/client-grants", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "audience": None,
                "page": 7,
                "per_page": 23,
                "include_totals": "true",
                "client_id": None,
            },
        )

        # With client_id param
        c.all(client_id="exampleid")

        args, kwargs = mock_instance.get.call_args

        self.assertEqual("https://domain/api/v2/client-grants", args[0])
        self.assertEqual(
            kwargs["params"],
            {
                "audience": None,
                "page": None,
                "per_page": None,
                "include_totals": "false",
                "client_id": "exampleid",
            },
        )

    @mock.patch("auth0.management.client_grants.RestClient")
    def test_create(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = ClientGrants(domain="domain", token="jwttoken")
        c.create({"a": "b", "c": "d"})

        mock_instance.post.assert_called_with(
            "https://domain/api/v2/client-grants", data={"a": "b", "c": "d"}
        )

    @mock.patch("auth0.management.client_grants.RestClient")
    def test_delete(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = ClientGrants(domain="domain", token="jwttoken")
        c.delete("this-id")

        mock_instance.delete.assert_called_with(
            "https://domain/api/v2/client-grants/this-id"
        )

    @mock.patch("auth0.management.client_grants.RestClient")
    def test_update(self, mock_rc):
        mock_instance = mock_rc.return_value

        c = ClientGrants(domain="domain", token="jwttoken")
        c.update("this-id", {"a": "b", "c": "d"})

        args, kwargs = mock_instance.patch.call_args

        self.assertEqual("https://domain/api/v2/client-grants/this-id", args[0])
        self.assertEqual(kwargs["data"], {"a": "b", "c": "d"})



from ..rest import RestClient


class Clients:
    """Auth0 applications endpoints

    Args:
        domain (str): Your Auth0 domain, e.g: 'username.auth0.com'

        token (str): Management API v2 Token

        telemetry (bool, optional): Enable or disable Telemetry
            (defaults to True)

        timeout (float or tuple, optional): Change the requests
            connect and read timeout. Pass a tuple to specify
            both values separately or a float to set both to it.
            (defaults to 5.0 for both)

        rest_options (RestClientOptions): Pass an instance of
            RestClientOptions to configure additional RestClient
            options, such as rate-limit retries.
            (defaults to None)
    """

    def __init__(
        self,
        domain,
        token,
        telemetry=True,
        timeout=5.0,
        protocol="https",
        rest_options=None,
    ):
        self.domain = domain
        self.protocol = protocol
        self.client = RestClient(
            jwt=token, telemetry=telemetry, timeout=timeout, options=rest_options
        )

    def _url(self, id=None):
        url = f"{self.protocol}://{self.domain}/api/v2/clients"
        if id is not None:
            return f"{url}/{id}"
        return url

    def all(
        self,
        fields=None,
        include_fields=True,
        page=None,
        per_page=None,
        extra_params=None,
    ):
        """Retrieves a list of all the applications.

        Important: The client_secret and encryption_key attributes can only be
        retrieved with the read:client_keys scope.

        Args:
           fields (list of str, optional): A list of fields to include or
              exclude from the result (depending on include_fields). Leave empty to
              retrieve all fields.

           include_fields (bool, optional): True if the fields specified are
              to be included in the result, False otherwise. Defaults to True.

           page (int): The result's page number (zero based). When not set,
              the default value is up to the server.

           per_page (int, optional): The amount of entries per page. When not set,
              the default value is up to the server.

           extra_params (dictionary, optional): The extra parameters to add to
             the request. The fields, include_fields, page and per_page values
             specified as parameters take precedence over the ones defined here.

        See: https://auth0.com/docs/api/management/v2#!/Clients/get_clients
        """
        params = extra_params or {}
        params["fields"] = fields and ",".join(fields) or None
        params["include_fields"] = str(include_fields).lower()
        params["page"] = page
        params["per_page"] = per_page

        return self.client.get(self._url(), params=params)

    def create(self, body):
        """Create a new application.

        Args:
           body (dict): Attributes for the new application.

        See: https://auth0.com/docs/api/v2#!/Clients/post_clients
        """

        return self.client.post(self._url(), data=body)

    def get(self, id, fields=None, include_fields=True):
        """Retrieves an application by its id.

        Important: The client_secret, encryption_key and signing_keys
        attributes can only be retrieved with the read:client_keys scope.

        Args:
           id (str): Id of the application to get.

           fields (list of str, optional): A list of fields to include or
              exclude from the result (depending on include_fields). Leave empty to
              retrieve all fields.

           include_fields (bool, optional): True if the fields specified are
              to be included in the result, False otherwise. Defaults to True.

        See: https://auth0.com/docs/api/management/v2#!/Clients/get_clients_by_id
        """

        params = {
            "fields": fields and ",".join(fields) or None,
            "include_fields": str(include_fields).lower(),
        }

        return self.client.get(self._url(id), params=params)

    def delete(self, id):
        """Deletes an application and all its related assets.

        Args:
           id (str): Id of application to delete.

        See: https://auth0.com/docs/api/management/v2#!/Clients/delete_clients_by_id
        """

        return self.client.delete(self._url(id))

    def update(self, id, body):
        """Modifies an application.

        Important: The client_secret, encryption_key and signing_keys
        attributes can only be updated with the update:client_keys scope.

        Args:
           id (str): Client ID of the application.

           body (dict): Attributes to modify.

        See: https://auth0.com/docs/api/management/v2#!/Clients/patch_clients_by_id
        """

        return self.client.patch(self._url(id), data=body)

    def rotate_secret(self, id):
        """Rotate a client secret. The generated secret is NOT base64 encoded.

        Args:
           id (str): Client ID of the application.

        See: https://auth0.com/docs/api/management/v2#!/Clients/post_rotate_secret
        """

        data = {"id": id}

        url = self._url("%s/rotate-secret" % id)
        return self.client.post(url, data=data)
